#include  "gtk_agent_option_analyzer.h"
#include  "viewer_option_descriptor.h"
#include  <utility>

int	Gtk_Agent_Option_Analyzer::DEFAULT_VALID_PLAYER_THRESHOLD = -60;
double	Gtk_Agent_Option_Analyzer::DEFAULT_BASE_PLAYER_SIZE = 1.3;
double	Gtk_Agent_Option_Analyzer::DEFAULT_PLAYER_SIZE_RATE = 0.04;


Gtk_Agent_Option_Analyzer::Gtk_Agent_Option_Analyzer
		 ( const std::string &  default_team_name ,
		   bool  default_goalie_flag ,
		   bool  default_coach_flag ,
		   const std::string &  default_host_name ,
		   port_number_t  default_port_number ,
		   bool  default_with_debug_server ,
		   const std::string &  default_debug_host_name ,
		   port_number_t  default_debug_port_number ,
		   bool  default_with_evolution_server ,
		   const std::string &  default_evolution_server_host_name ,
		   port_number_t  default_evolution_server_port_number ,
		   bool  default_with_file_repository ,
		   const std::string &  default_file_repository_path ,
		   int  default_server_major_version ,
		   int  default_server_minor_version )
	 : Soccer_Option_Analyzer( default_team_name ,
				   default_goalie_flag ,
				   default_coach_flag ,
				   default_host_name ,
				   default_port_number ,
				   default_with_debug_server ,
				   default_debug_host_name ,
				   default_debug_port_number ,

				   default_with_evolution_server ,
				   default_evolution_server_host_name ,
				   default_evolution_server_port_number ,
				   default_with_file_repository ,
				   default_file_repository_path ,
				   default_server_major_version ,
				   default_server_minor_version ) ,
	   valid_pl_threshold( DEFAULT_VALID_PLAYER_THRESHOLD ) ,
	   base_pl_size( DEFAULT_BASE_PLAYER_SIZE ) ,
	   pl_size_rate( DEFAULT_PLAYER_SIZE_RATE ) ,
	   fixed_pl_size( false ) ,
	   burn_out_flag( true ) ,
	   reverse_x_( false ) ,
	   reverse_y_( false ) ,
	   field_grass_type_( Field_Canvas::DEFAULT_GRASS_TYPE ) ,
	   window_size( 0 , 0 )
{
	this -> add_long_option_integer ( "--valid-player-threshold" ,
					  &valid_pl_threshold );
	this -> add_long_option_float   ( "--base-player-size" ,
					  &base_pl_size );
	this -> add_long_option_float   ( "--player-size-rate" ,
					  &pl_size_rate );
	this -> add_long_option_flag_on ( "--fixed-player-size" ,
					  &fixed_pl_size );
	this -> add_long_option_flag_off( "--no-burn-out" ,
					  &burn_out_flag );

	this -> add_long_option_flag_on(  "--reverse-x" ,
					  &reverse_x_ );
	this -> add_long_option_flag_on(  "--reverse-y" ,
					  &reverse_y_ );
	this -> add_long_option
		( "--field-grass-type" ,
		  new Field_Grass_Type_Option_Descriptor
					      ( &field_grass_type_ ) );

	this -> add_long_option
		( "--default-window-size" ,
		  new Window_Size_Option_Descriptor( &window_size ) );
}

Gtk_Agent_Option_Analyzer::~Gtk_Agent_Option_Analyzer()
{
}

void   Gtk_Agent_Option_Analyzer::usage( std::ostream *  ostr ) const
{
	this -> Soccer_Option_Analyzer::usage( ostr );

	(*ostr) << "      --valid-player-threshold STEP (default "
		<< DEFAULT_VALID_PLAYER_THRESHOLD << " step)" << std::endl
		<< "      --base-player-size SIZE (default "
		<< DEFAULT_BASE_PLAYER_SIZE << " meter)" << std::endl
		<< "      --player-size-rate RATE (default "
		<< DEFAULT_PLAYER_SIZE_RATE << " meter/step)" << std::endl
		<< "      --fixed-player-size" << std::endl
		<< "      --no-burn-out" << std::endl
		<< "      --reverse-x" << std::endl
		<< "      --reverse-y" << std::endl
		<< "      --field-grass-type {mono,lines,checker}" << std::endl
		<< "      --default-window-size WIDTHxHEIGHT" << std::endl
		<< " and gtk options (--display etc.)" << std::endl;
}

int    Gtk_Agent_Option_Analyzer::valid_player_threshold() const
{
	return( valid_pl_threshold );
}

double  Gtk_Agent_Option_Analyzer::base_player_size() const
{
	return( this -> base_pl_size );
}

double  Gtk_Agent_Option_Analyzer::player_size_rate() const
{
	if ( this -> fixed_pl_size )
	{
		return( 0.0 );
	}
	else
	{
		return( this -> pl_size_rate );
	}
}

bool   Gtk_Agent_Option_Analyzer::burn_out() const
{
	return( this -> burn_out_flag );
}

bool   Gtk_Agent_Option_Analyzer::reverse_x() const
{
	return( this -> reverse_x_ );
}

bool   Gtk_Agent_Option_Analyzer::reverse_y() const
{
	return( this -> reverse_y_ );
}

Field_Canvas::Field_Grass_Type  Gtk_Agent_Option_Analyzer
						::field_grass_type() const
{
	return( this -> field_grass_type_ );
}

std::pair<int, int>  Gtk_Agent_Option_Analyzer::default_window_size() const
{
	return( this -> window_size );
}
