#ifndef	   EVOLUTIONAL_VARIABLE_H_INCLUDED
#define	   EVOLUTIONAL_VARIABLE_H_INCLUDED

// Author:		H. Shimora
// Created:		Dec 18 2001
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Dec 18 2001    base version.
//
//

#include  "evolutional_repository.h"
#include  "ref_count_ptr.h"
#include  "string_extension.h"
#include  <string>
#include  <iostream>


class  Evolutional_Variable
{
private:
	ref_count_ptr<Evolutional_Repository>	rep;
	std::string				name;
	std::string				id;
	bool					evaluated;

protected:
	virtual	void	get_value( const std::string &  name ,
				   std::string *  str_val )
	{
		this -> rep -> get_value( name , str_val , &(this -> id) );
	}

public:
	Evolutional_Variable
		( const ref_count_ptr<Evolutional_Repository> &  rep ,
		  const std::string &  name )
			: rep( rep ) ,
			  name( name ) , id( "" ) ,
			  evaluated( false )
	{
	}

	virtual	~Evolutional_Variable()
	{
		if ( ! (this -> evaluated) )
		{
			this -> rep -> evaluation_cancel
					( this -> name , this -> id );
		}
	}

	virtual	void	evaluate( double  evaluation_value )
	{
		this -> rep -> evaluate( this -> name , this -> id ,
					 evaluation_value );
		this -> evaluated = true;
	}
};


class  Evolutional_Double : public Evolutional_Variable
{
private:
	double	value;

public:
	 Evolutional_Double
		 ( const ref_count_ptr<Evolutional_Repository> &  rep ,
		   const std::string &  name )
			 : Evolutional_Variable( rep , name ) ,
			   value( 0.0 )
	{
		std::string	str_val;
		this -> get_value( name , &str_val );

		if ( ! String_Extension::string_to_double
					     ( &(this -> value) , str_val ) )
		{
			std::cerr << str_val
				  << ": invalid value, real number requested."
				  << std::endl;

			this -> value = 0.0;
		}
	}

	~Evolutional_Double() {}

	operator double () const
	{
		return( value );
	}
};


#endif	/* EVOLUTIONAL_VARIABLE_H_INCLUDED */
