#ifndef	   INTERACTIVE_EVOLUTIONAL_REPOSITORY_H_INCLUDED
#define	   INTERACTIVE_EVOLUTIONAL_REPOSITORY_H_INCLUDED

// Author:		H. Shimora
// Created:		Sep 27 2002
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Sep 27 2002    base version.
//
//

#include  "evolutional_repository.h"
#include  "stream_socket_wrapper.h"
#include  "ip_address.h"
#include  "string_extension.h"
#include  <string>
#include  <csignal>
#include  <cstdio>
#include  <iostream>

class  Interactive_Evolutional_Repository : public Evolutional_Repository
{
private:
	Stream_Socket_Wrapper	socket;

public:
	Interactive_Evolutional_Repository( const std::string &  host ,
					    const port_number_t &  port )
		: socket( new Stream_Socket( AF_INET , SOCK_STREAM ) )
	{
		// XXX
		if ( std::signal( SIGPIPE , SIG_IGN ) == SIG_ERR )
		{
			std::perror( "signal" );

			return;
		}

		if ( ! this -> socket.valid()
		  || this -> socket.socket() -> connect( host , port ) == -1 )
		{
			return;
		}
	}

	virtual	~Interactive_Evolutional_Repository()
	{
	}

	virtual	void	get_value( const std::string &  name ,
				   std::string *  value ,
				   std::string *  id )
	{
		*value = "";
		*id = "";

		if ( ! this -> socket.connected() )
		{
			return;
		}

		this -> socket.send_message( "project " + name + " "
					     + "get-sample\n" );
		// XXX
		std::cerr << "<< project " + name + " get-sample" << std::endl;

		std::string	result_string;
		this -> socket.recv_message( &result_string );

		// XXX
		std::cerr << ">> " << result_string << std::endl;

		if ( result_string.length() >= 1
		  && result_string[0] == '#' )
		{
			std::cerr << "evolutional server error: "
				  << result_string.substr( 1 ) << std::endl;
			return;
		}


		std::vector<std::string>	result;
		String_Extension::split_by_char_set( &result ,
						     result_string , " \t" );

		if ( result.size() < 2 )
		{
			std::cerr << "evolutional server format mismatch: "
				  << result_string << std::endl;
			return;
		}

		*id    = result[0];
		*value = result[1];
	}

	virtual	void	evaluate( const std::string &  name ,
				  const std::string &  id ,
				  double  evaluation_value )
	{
		if ( ! this -> socket.connected()
		  || id.length() == 0 )
		{
			return;
		}


		std::string	evaluation_value_string;
		String_Extension::double_to_string( &evaluation_value_string ,
						    evaluation_value );

		this -> socket.send_message( "project " + name + " "
					     + "evaluation-result "
					     + id + " "
					     + evaluation_value_string
					     + "\n" );

		std::cerr << "<< project " + name + " "
					   + "evaluation-result "
					   + id + " "
					   + evaluation_value_string
			  << std::endl;

		std::string	result_string;
		this -> socket.recv_message( &result_string );
		// XXX
		std::cerr << ">> " << result_string << std::endl;
	}

	virtual	void	evaluation_cancel( const std::string &  name ,
					   const std::string &  id )
	{
		if ( ! this -> socket.connected()
		  || id.length() == 0 )
		{
			return;
		}


		this -> socket.send_message( "project " + name + " "
					     + "evaluation-cancel "
					     + id + "\n" );

		// XXX
		std::cerr << "<< project " + name + " "
					   + "evaluation-cancel "
					   + id << std::endl;

		std::string	result_string;
		this -> socket.recv_message( &result_string );
		// XXX
		std::cerr << ">> " << result_string << std::endl;
	}
};


#endif	/* INTERACTIVE_EVOLUTIONAL_REPOSITORY_H_INCLUDED */
