#include  <iostream>
#include  <cstdlib>
#include  <string>
#include  <cmath>

template<class T>
void   Assert_Equals( const T &  a ,  const T &  b)
{
	if ( a != b )
	{
		std::cerr << "[" << b << "] should be [" << a << "]"
			  << std::endl;

		std::exit( 1 );
	}
}

void   Assert_Equals( const std::string &  a ,  const char *  b )
{
	Assert_Equals( a , std::string(b) );
}

void   Assert_Equals( const char *  a , const std::string &  b )
{
	Assert_Equals( std::string(a) , b );
}


void   Assert_Nearly_Equals( double  a ,  double  b )
{
	if ( std::fabs( a - b ) > 1.0e-10 )
	{
		std::cerr << "[" << b << "] should be nearly [" << a << "]"
			  << std::endl;

		std::exit( 1 );
	}
}



#include  "compat_stringstream.h"
#include  <iostream>

int    main( void )
{
	{
		compat_stringstream	s;

		s << 1 << "hello, world" << std::endl;

		Assert_Equals( "1hello, world\n" , s.str() );

		char	one;
		s >> one;

		Assert_Equals( '1' , one );
		Assert_Equals( "hello, world\n" , s.str() );


		std::string	hello;
		std::getline( s , hello );
		Assert_Equals( "hello, world" , hello );
	}

	{
		compat_stringstream	s( "3.14" );
		double	value = 0.0;
		s >> value;

		Assert_Nearly_Equals( 3.14 , value );
	}

	{
		compat_stringstream	s;
		s << 1.23;

		double	value;
		s >> value;

		Assert_Nearly_Equals( 1.23 , value );
	}


	return( 0 );
}
