#ifndef	   COMPAT_STRINGSTREAMBUF_H_INCLUDED
#define	   COMPAT_STRINGSTREAMBUF_H_INCLUDED

#ifdef HAVE_CONFIG_H
# include  "config.h"
#endif

#include  "streambuf_compat_wrapper.h"
#include  <string>
#include  <cstdio>


class  compat_stringstreambuf : public std::streambuf
{
private:
	char *	input_buffer_begin() const
	{
		return( this -> std::streambuf::eback() );
	}

	char *	input_buffer_end() const
	{
		return( this -> std::streambuf::egptr() );
	}

	char *	input_buffer_current() const
	{
		return( this -> std::streambuf::gptr() );
	}

	void	input_buffer_move_current( int  n )
	{
		this -> gbump( n );
	}

	char *	output_buffer_begin() const
	{
		return( this -> std::streambuf::pbase() );
	}

	char *	output_buffer_end() const
	{
		return( this -> std::streambuf::epptr() );
	}

	char *	output_buffer_current() const
	{
		return( this -> std::streambuf::pptr() );
	}

	void	output_buffer_move_current( int  n )
	{
		this -> pbump( n );
	}


private:
	 // not allowed.
	 compat_stringstreambuf( const compat_stringstreambuf & );

	 // not allowed.
	 compat_stringstreambuf &
		     operator= ( const compat_stringstreambuf & );


private:
	char		buffer[BUFSIZ];
	std::string	io_str;


	void	initialize_buffer()
	{
		this -> setg( buffer , buffer , buffer );
	}

public:
		compat_stringstreambuf()
			: io_str()
		{
			this -> initialize_buffer();
		}

		compat_stringstreambuf( const std::string &  input )
			: io_str( input )
		{
			this -> initialize_buffer();
		}

	virtual	~compat_stringstreambuf()
		{
		}

	virtual	int  overflow( int  ch = EOF )
		{
			if ( ch == EOF )
			{
				return( EOF );
			}

			this -> io_str.append( 1 , static_cast<char>(ch) );

			return( ch );
		}

	virtual	int  underflow()
		{
			int	ch = this -> uflow();

			this -> pbackfail( ch );

			return( ch );
		}

	virtual	int  uflow()
		{
			if ( this -> input_buffer_current()
			     >= this -> buffer + sizeof(this -> buffer) )
			{
				this -> setg
					( this -> input_buffer_begin() ,
					  this -> input_buffer_begin() ,
					  this -> input_buffer_begin() );
			}

			if ( ! this -> io_str.empty() )
			{
				int	ch = io_str[0];

				*(this -> input_buffer_current()) = ch;

				this -> io_str.erase( this -> io_str.begin() );

				this -> setg
					( this -> input_buffer_begin() ,
					  this -> input_buffer_current() ,
					  this -> input_buffer_current() + 1 );
				this -> input_buffer_move_current( 1 );

				return( ch );
			}
			else
			{
				return( EOF );
			}
		}

	virtual	int	pbackfail( int  ch )
		{
			if ( this -> input_buffer_current()
			     > this -> input_buffer_begin() )
			{
				this -> input_buffer_move_current( -1 );

				return( ch );
			}
			else
			{
				return( EOF );
			}

			return( ch );
		}

	virtual	const std::string &	str() const
		{
			compat_stringstreambuf *
			    t = const_cast<compat_stringstreambuf *>( this );

			int	n = this -> output_buffer_end()
				    - this -> output_buffer_current();

			t -> io_str.append( this -> output_buffer_current() ,
					    n );

			t -> output_buffer_move_current( n );

			return( this -> io_str );
		}
};


#endif	/* COMPAT_STRINGSTREAMBUF_H_INCLUDED */
