#ifndef	   COMMANDBUF_H_INCLUDED
#define	   COMMANDBUF_H_INCLUDED

#ifdef HAVE_CONFIG_H
# include  "config.h"
#endif

#include  "streambuf_compat_wrapper.h"
#include  <cstdio>
#include  <string>
#include  <vector>
#include  <csignal>
#include  <cstdlib>
#include  <unistd.h>
#include  <errno.h>
#include  <sys/types.h>
#include  <sys/wait.h>


class  commandbuf : public std::streambuf
{
public:
	enum { BAD_PROCESSS_ID = -1 };
	enum { BAD_FD = -1 };

protected:
	int	to_com_fd;
	int	from_com_fd;
	pid_t	command_pid;

private:
	char	buffer[BUFSIZ];

	void	initialize_buffer()
	{
		this -> setg( buffer , buffer , buffer );
	}

private:
	char *	input_buffer_begin() const
	{
		return( this -> std::streambuf::eback() );
	}

	char *	input_buffer_end() const
	{
		return( this -> std::streambuf::egptr() );
	}

	char *	input_buffer_current() const
	{
		return( this -> std::streambuf::gptr() );
	}

	void	input_buffer_move_current( int  n )
	{
		this -> gbump( n );
	}

public:
	void	invoke( const char *  file ,
			const char * const * const  argv );

	void	invoke( const std::string &  file ,
			const std::vector<std::string> &  argv );

private:
	 commandbuf( const commandbuf & );			// not allowed.
	 commandbuf &	operator= ( const commandbuf & );	// not allowed.

public:
	 commandbuf();

	 commandbuf( const std::string &  file ,
		     const std::vector<std::string> &  argv );
	 commandbuf( const std::vector<std::string> &  argv );

	 commandbuf( const char *  file ,  const char * const * const  argv );
	 commandbuf( const char * const * const  argv );

	virtual	~commandbuf();

protected:
	virtual	int  overflow( int  ch = EOF )
		{
			if ( ch == EOF || to_com_fd == BAD_FD )
			{
				return( EOF );
			}

			unsigned char	c = static_cast<unsigned char>(ch);

			if ( ::write( to_com_fd , &c ,
				      static_cast<size_t>(1) ) != 1 )
			{
				return( EOF );
			}
			else
			{
				return( ch );
			}
		}

	virtual	int  underflow()
		{
			int	ch = this -> uflow();

			this -> pbackfail( ch );

			return( ch );
		}

	virtual	int  uflow()
		{
			if ( from_com_fd == BAD_FD )
			{
				return( EOF );
			}

			if ( this -> input_buffer_current()
			     >= this -> buffer + sizeof(this -> buffer) )
			{
				this -> setg
					( this -> input_buffer_begin() ,
					  this -> input_buffer_begin() ,
					  this -> input_buffer_begin() );
			}

			const size_t	free_buf_size
					= (this -> buffer
					   + sizeof(this -> buffer))
					  - this -> input_buffer_current();

			int	n;

			switch( (n = ::read( from_com_fd ,
					     this -> input_buffer_current() ,
					     free_buf_size )) )
			{
			case -1:
				std::perror( "read" );
				return( EOF );
				break;

			case  0:
				return( EOF );
				break;

			default:
			      {
				unsigned char	ch;
				ch = *(this -> input_buffer_current());

				this -> setg
					( this -> input_buffer_begin() ,
					  this -> input_buffer_current() ,
					  this -> input_buffer_current() + n );
				this -> input_buffer_move_current( 1 );

				return( ch );
				break;
			      }
			}
		}

	virtual	int	pbackfail( int  ch )
		{
			if ( this -> input_buffer_current()
			     > this -> input_buffer_begin() )
			{
				this -> input_buffer_move_current( -1 );

				return( ch );
			}
			else
			{
				return( EOF );
			}
		}


public:
	virtual	void  close( int  in_out = std::ios::in | std::ios::out )
		{
			if ( in_out & std::ios::in )
			{
				if ( to_com_fd != BAD_FD
				  && ::close( to_com_fd ) == -1 )
				{
					std::perror( "close" );
				}

				to_com_fd = BAD_FD;

				this -> initialize_buffer();
			}

			if ( in_out & std::ios::out )
			{
				if ( from_com_fd != BAD_FD
				  && ::close( from_com_fd ) == -1 )
				{
					std::perror( "close" );
				}

				from_com_fd = BAD_FD;
			}
		}

	virtual	int	fd( int  in_out ) const
		{
			if ( in_out == std::ios::in )
			{
				return( to_com_fd );
			}
			else if ( in_out == std::ios::out )
			{
				return( from_com_fd );
			}
			else
			{
				return( BAD_FD );
			}
		}

	virtual	pid_t	getpid() const { return( command_pid ); }

	virtual	int	kill( int  sig = SIGTERM )
		{
			if ( command_pid == BAD_PROCESSS_ID )
			{
				return( -1 );
			}
			else
			{
				int	ret;

				if ( (ret = ::kill( command_pid , sig )) == -1
				  && errno != ESRCH /* No such process */ )
				{
					std::perror( "kill" );
				}

				return( ret );
			}
		}

	virtual	int	poll() const;
};



inline
void   commandbuf::invoke( const char *  file ,
			   const char * const * const  argv )
{
	this -> close( std::ios::in | std::ios::out );
	this -> kill();

	to_com_fd   = BAD_FD;
	from_com_fd = BAD_FD;
	command_pid = BAD_PROCESSS_ID;

	int	to  [2];
	int	from[2];

	if ( ::pipe( to ) == -1 )
	{
		std::perror( "pipe" );

		return;
	}

	if ( ::pipe( from ) == -1 )
	{
		std::perror( "pipe" );

		if ( ::close( to[0] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( to[1] ) == -1 ) { std::perror( "close" ); }

		return;
	}


	switch( (command_pid = ::fork()) )
	{
	case  -1:
		// error
		std::perror( "fork" );

		if ( ::close(   to[0] ) == -1 )	{ std::perror( "close" ); }
		if ( ::close(   to[1] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( from[0] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( from[1] ) == -1 ) { std::perror( "close" ); }

		return;
		break;

	case  0:
		// child process
		if ( ::close(   to[1] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( from[0] ) == -1 ) { std::perror( "close" ); }

		if ( ::dup2(   to[0] , STDIN_FILENO  ) == -1 )
		{
			std::perror( "dup2" );
		}

		if ( ::dup2( from[1] , STDOUT_FILENO ) == -1 )
		{
			std::perror( "dup2" );
		}

		if ( ::close(   to[0] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( from[1] ) == -1 ) { std::perror( "close" ); }

		if ( ::execvp( file , const_cast<char * const *>(argv) )
		     == -1 )
		{
			std::perror( "execvp" );
		}

		::_exit( 1 );

		break;

	default:
		// parent process
		if ( ::close(   to[0] ) == -1 ) { std::perror( "close" ); }
		if ( ::close( from[1] ) == -1 ) { std::perror( "close" ); }

		to_com_fd   = to  [1];
		from_com_fd = from[0];

		break;
	}
}

inline
void   commandbuf::invoke( const std::string &  file ,
			   const std::vector<std::string> &  argv )
{
	const char **	v = static_cast<const char **>(0);

	try
	{
		v = new const char * [argv.size() + 1];

		size_t  i;
		for ( i = 0  ;  i < argv.size()  ;  i ++ )
		{
			v[i] = argv[i].c_str();
		}

		v[i] = static_cast<char *>(0);

		this -> invoke( file.c_str() , v );
	}
	catch(...)
	{
		delete[] v;

		throw;
	}

	delete[] v;
}

inline
commandbuf::commandbuf()
	: to_com_fd( BAD_FD ) , from_com_fd( BAD_FD ) ,
	  command_pid( BAD_PROCESSS_ID )
{
	this -> initialize_buffer();
}

inline
commandbuf::commandbuf( const std::string &  file ,
			const std::vector<std::string> &  argv )
	: to_com_fd( BAD_FD ) , from_com_fd( BAD_FD ) ,
	  command_pid( BAD_PROCESSS_ID )
{
	this -> initialize_buffer();

	this -> invoke( file , argv );
}

inline
commandbuf::commandbuf( const std::vector<std::string> &  argv )
	: to_com_fd( BAD_FD ) , from_com_fd( BAD_FD ) ,
	  command_pid( BAD_PROCESSS_ID )
{
	this -> initialize_buffer();

	this -> invoke( argv[0] , argv );
}

inline
commandbuf::commandbuf( const char *  file ,
			const char * const * const  argv )
	: to_com_fd( BAD_FD ) , from_com_fd( BAD_FD ) ,
	  command_pid( BAD_PROCESSS_ID )
{
	this -> initialize_buffer();

	this -> invoke( file , argv );
}

inline
commandbuf::commandbuf( const char * const * const  argv )
	: to_com_fd( BAD_FD ) , from_com_fd( BAD_FD ) ,
	  command_pid( BAD_PROCESSS_ID )
{
	this -> initialize_buffer();

	this -> invoke( argv[0] , argv );
}


inline
commandbuf::~commandbuf()
{
	this -> close( std::ios::in | std::ios::out );

	if ( command_pid != BAD_PROCESSS_ID
	  && command_pid != 0 )
	{
		if ( ::waitpid( command_pid ,
				static_cast<int *>(0) ,
				0  ) == -1 )
		{
			std::perror( "waitpid" );
		}
	}
}



#include  <unistd.h>
#include  <sys/types.h>
#include  <cstring>

#if defined(TIME_WITH_SYS_TIME) && defined(HAVE_SYS_TIME_H)
    #if TIME_WITH_SYS_TIME
	#include  <sys/time.h>
	#include  <time.h>
    #else
	#if HAVE_SYS_TIME_H
	    #include  <sys/time.h>
	#else
	    #include  <time.h>
	#endif
    #endif
#else
    #include  <sys/time.h>
#endif

inline
int    commandbuf::poll() const
{
	if ( (this -> input_buffer_end()
	      - this -> input_buffer_current()) > 0 )
	{
		return( 1 );
	}


	fd_set		readfd;
	FD_ZERO( &readfd );
	FD_SET( this -> from_com_fd , &readfd );

	struct timeval	tv;
	tv.tv_sec  = 0;
	tv.tv_usec = 0;

	int	ret;

	if ( (ret = ::select( this -> from_com_fd + 1 , &readfd ,
			      static_cast<fd_set *>(0) ,
			      static_cast<fd_set *>(0) ,
			      &tv ) ) == -1 )
	{
		std::perror( "select" );

		return( 0 );
	}

	return( ret );
}


#endif	/* COMMANDBUF_H_INCLUDED */
