#include  "comstream.h"
#include  <cassert>
#include  <string>
#include  <cmath>


int    main( void )
{
	// grep
	{
		comstream	com( "grep" , "^a" , static_cast<char *>(0) );

		assert( com );

		com << "abcdefg" << std::endl;
		com << "abc" << std::endl;
		com << "def" << std::endl;
		com << "ghi" << std::endl;
		com << "aaa" << std::endl;
		com.close( std::ios::in );

		std::string	result;

		for(;;)
		{
			std::string	line;

			std::getline( com , line );

			if ( ! com )
			{
				break;
			}

			result += line;
		}

		assert( result == "abcdefgabcaaa" );
	}


	// get double test
	{
		icomstream	com( "echo" , "3.14 2.7" ,
				     static_cast<char *>(0) );

		assert( com );

		double	a = 0.0;
		com >> a;

		assert( ! com.eof() );

		double	b = 0.0;
		com >> b;

		assert( std::fabs( a - 3.14 ) < 1.0e-10 );
		assert( std::fabs( b - 2.7  ) < 1.0e-10 );
	}


	// read test
	{
		std::string	input = "abcdefghij";

		icomstream	com( "echo" ,
				     "-n" ,
				     input.c_str() ,
				     static_cast<char *>(0) );

		assert( com );

		std::string	result;

		while( com )
		{
			char	buf[4];

			if ( result.length() < input.length() )
			{
				assert( ! com.eof() );
			}

			com.read( buf , sizeof(buf) );

			if ( com.gcount() > 0 )
			{
				result.append( buf , com.gcount() );
			}
		}

		assert( result == input );
	}


	// eof test
	{
		std::string	input = "1234567890";

		icomstream	com( "echo" ,
				     "-n" ,
				     input.c_str() ,
				     static_cast<char *>(0) );

		assert( com );

		std::string	result;
		for ( size_t  i = 0  ;  i < input.size()  ;  i ++ )
		{
			result += com.get();

			assert( com );
		}

		assert( ! com.eof() );
		com.get();
		assert( com.eof() );

		assert( result == input );
	}

	// large input test
	{
		comstream	com( "cat" ,
				     static_cast<char *>(0) );

		assert( com );

		const int	N = 10000;

		for ( int  i = 0  ;  i < N  ;  i ++ )
		{
			com.put( 'a' );

			int	ch = com.get();

			assert( ch == 'a' );

			assert( com );
		}

		com.close( std::ios::in );

		assert( ! com.eof() );
		com.get();
		assert( com.eof() );
	}


	// large double input test
	{
		comstream	com( "cat" ,
				     static_cast<char *>(0) );

		assert( com );

		const int	N = 10000;

		for ( int  i = 0  ;  i < N  ;  i ++ )
		{
			com << i + i * 0.1 << std::endl;

			double	value;
			com >> value;

			assert( value - (i + i * 0.1) < 1.0e-10 );

			assert( com );
		}
		com.close( std::ios::in );

		assert( ! com.eof() );
		double	value;
		com >> value;
		assert( com.eof() );
	}


	return( 0 );
}


#if 0
#include  "comstream.h"
#include  <iostream>
#include  <string>

using namespace std;

int    main( void )
{
	comstream	com( "grep" , "^a" , static_cast<char *>(0) );

	if ( com )
	{
		com << "abcdefg" << endl;
		com << "abc" << endl;
		com << "def" << endl;
		com << "ghi" << endl;
		com << "aaa" << endl;
		com.close( ios::in );

		for(;;)
		{
			string	line;

			getline( com , line );

			if ( ! com )
			{
				break;
			}

			cout << line << endl;
		}
	}

	return( 0 );
}
#endif
