#ifndef	   OPTION_ANALYZER_H_INCLUDED
#define	   OPTION_ANALYZER_H_INCLUDED

// Author:		H. Shimora
// Created:		Jan 27 2000
// Last-Modified:	Jan 27 2000
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Jan 27 2000    test version.
//
//

#ifdef HAVE_CONFIG_H
# include  "config.h"
#endif

#include  <string>
#include  <vector>
#include  <map>
#include  <cstdlib>
#include  <iostream>
#include  "ref_count_ptr.h"
#include  "ip_address.h"

class  Option_Analyzer
{
public:
	class  Option_Descriptor
	{
	public:
		virtual	~Option_Descriptor() {}

		virtual	int	execute( const std::vector<std::string> & )
									const
		{
			return( 0 );
		}

		virtual	bool	check( std::string * ) const
		{
			return( true );
		}
	};

public:
	class  Flag_On_Option_Descriptor : public Option_Descriptor
	{
	protected:
		bool *	flag;

	public:
			 Flag_On_Option_Descriptor( bool *  f )
				 : flag( f ) {}

		virtual	~Flag_On_Option_Descriptor() {}

		virtual	int	execute( const std::vector<std::string> & )
									const
		{
			if ( flag )
			{
				*flag = true;
			}

			return( 0 );
		}
	};

	class  Flag_Off_Option_Descriptor : public Option_Descriptor
	{
	protected:
		bool *	flag;

	public:
			 Flag_Off_Option_Descriptor( bool *  f )
				 : flag( f ) {}

		virtual	~Flag_Off_Option_Descriptor() {}

		virtual	int	execute( const std::vector<std::string> & )
									const
		{
			if ( flag )
			{
				*flag = false;
			}

			return( 0 );
		}
	};

	class  Flag_Toggle_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		bool *	flag;

	public:
			 Flag_Toggle_Option_Descriptor( bool *  f )
				 : flag( f ) {}
		virtual	~Flag_Toggle_Option_Descriptor() {}

		virtual	int	execute( const std::vector<std::string> & )
									const
		{
			if ( flag )
			{
				*flag = ! *flag;
			}

			return( 0 );
		}
	};

	class  Integer_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		int *	integer;

	public:
			 Integer_Option_Descriptor( int *  i )
				 : integer( i ) {}

		virtual	~Integer_Option_Descriptor() {}

		virtual	int	execute
				( const std::vector<std::string> &  arg ) const
		{
			if ( arg.size() < 1 )
			{
				return( -1 );
			}

			if ( integer )
			{
				char *	c;
				long	long_num;
				long_num
				    = std::strtol( arg[0].c_str() , &c , 10 );

				if ( *c != '\0'
				  || long_num == LONG_MIN
				  || long_num == LONG_MAX
				  || long_num < INT_MIN
				  || long_num > INT_MAX )
				{
					std::cerr << "integer number option"
						  << " error ["
						  << arg[0] << "]"
						  << std::endl;

					return( -1 );
				}
				else
				{
					*integer = static_cast<int>(long_num);
				}
			}

			return( 1 );
		}
	};

	class  Long_Integer_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		long *	integer;

	public:
			 Long_Integer_Option_Descriptor( long *  i )
				 : integer( i ) {}
		virtual	~Long_Integer_Option_Descriptor() {}

		virtual	int	execute
				( const std::vector<std::string> &  arg ) const
		{
			if ( arg.size() < 1 )
			{
				return( -1 );
			}

			if ( integer )
			{
				char *	c;
				long	long_num;
				long_num
				    = std::strtol( arg[0].c_str() , &c , 10 );

				if ( *c != '\0'
				  || long_num == LONG_MIN
				  || long_num == LONG_MAX
				  || long_num < INT_MIN
				  || long_num > INT_MAX )
				{
					std::cerr << "integer number option"
						  << " error ["
						  << arg[0] << "]"
						  << std::endl;

					return( -1 );
				}
				else
				{
					*integer = long_num;
				}
			}

			return( 1 );
		}
	};

	class  Float_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		double *	number;

	public:
			 Float_Option_Descriptor( double *  n )
				 : number( n ) {}
		virtual	~Float_Option_Descriptor() {}

		virtual	int	execute
				( const std::vector<std::string> &  arg ) const
		{
			if ( arg.size() < 1 )
			{
				return( -1 );
			}

			if ( number )
			{
				char *	c;
				double	dbl_num;
				dbl_num = std::strtod( arg[0].c_str() , &c );

				if ( *c != '\0' )
				{
					std::cerr
					     << "float point number option"
					     << " error [" << arg[0] << "]"
					     << std::endl;

					return( -1 );
				}
				else
				{
					*number = dbl_num;
				}
			}

			return( 1 );
		}
	};

	class  String_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		std::string *	str;

	public:
			 String_Option_Descriptor( std::string *  s )
				 : str( s ) {}
		virtual	~String_Option_Descriptor() {}

		virtual	int	execute
				( const std::vector<std::string> &  arg ) const
		{
			if ( arg.size() < 1 )
			{
				return( -1 );
			}

			if ( str )
			{
				*str = arg[0];
			}

			return( 1 );
		}
	};

	class  Port_Number_Option_Descriptor
		: public Option_Analyzer::Option_Descriptor
	{
	protected:
		port_number_t *		port;

	public:
			 Port_Number_Option_Descriptor( port_number_t *  p )
				 : port( p ) {}

		virtual	~Port_Number_Option_Descriptor() {}

		virtual	int	execute
				( const std::vector<std::string> &  arg ) const
		{
			if ( arg.size() < 1 )
			{
				return( -1 );
			}

			if ( port )
			{
				char *	c;
				long	long_num;
				long_num
				    = std::strtol( arg[0].c_str() , &c , 10 );

				if ( *c != '\0'
				  || long_num < 0
				  || long_num > 0xffff )
				{
					std::cerr << "port number option"
						  << " error ["
						  << arg[0] << "]"
						  << std::endl;

					return( -1 );
				}
				else
				{
					*port = static_cast<port_number_t>
								    (long_num);
				}
			}

			return( 1 );
		}
	};

private:
	std::map< int , ref_count_ptr<const Option_Descriptor> >
							short_option_map;

	std::map< std::string , ref_count_ptr<const Option_Descriptor> >
							long_option_map;

	bool				valid;
	bool				more_options;

	std::string			argv_0;
	std::vector< std::string >	argv_original;
	std::vector< std::string >	rest_opt;

protected:
	virtual	void	finish_analyze();

protected:
	virtual	int	analyze_non_option( const std::vector<std::string> & );

	virtual	void	after_analyze_hook();

	virtual	bool	check() const;

protected:
	virtual	std::vector<std::string> &	rest();

public:
		 Option_Analyzer();
	virtual	~Option_Analyzer();

	virtual	void	usage( std::ostream *  ostr = &std::cerr ) const;

	virtual	bool	analyze( int  argc ,  const char *const *const  argv );
	virtual	bool	analyze( const std::vector<std::string> &  argv );

	virtual	const std::string &			program_name() const;

	virtual	const std::vector<std::string> &	argv() const;

	virtual	const std::vector<std::string> &	rest_options() const;

	virtual	operator  bool() const;


	//
	// Add Option
	//
	virtual	void	add_short_option
			( int  short_option ,
			  const ref_count_ptr<const Option_Descriptor> & opt );

	virtual	void	add_long_option
			( const std::string &  long_option ,
			  const ref_count_ptr<const Option_Descriptor> & opt );

	virtual	void	add_option
			( int  short_option ,
			  const std::string &  long_option ,
			  const ref_count_ptr<const Option_Descriptor> & opt );


	//
	// Add Flag On Option
	//
	virtual	void	add_short_option_flag_on( int  short_option ,
						  bool *  flag );

	virtual	void	add_long_option_flag_on
					( const std::string &  long_option ,
					  bool *  flag );

	virtual	void	add_option_flag_on( int  short_option ,
					    const std::string &  long_option ,
					    bool *  flag );


	//
	// Add Flag Off Option
	//
	virtual	void	add_option_flag_off( int  short_option ,
					     const std::string &  long_option ,
					     bool *  flag );

	virtual	void	add_short_option_flag_off( int  short_option ,
						   bool *  flag );

	virtual	void	add_long_option_flag_off
					( const std::string &  long_option ,
					  bool *  flag );

	//
	// Add Flag Toggle
	//
	virtual	void	add_short_option_flag_toggle( int  short_option ,
						      bool *  flag );

	virtual	void	add_long_option_flag_toggle
					( const std::string &  long_option ,
					  bool *  flag );

	virtual	void	add_option_flag_toggle
					( int  short_option ,
					  const std::string &  long_option ,
					  bool *  flag );

	//
	// Add Integer Option
	//
	virtual	void	add_short_option_integer( int  short_option ,
						  int *  i );

	virtual	void	add_long_option_integer
					( const std::string &  long_option ,
					  int *  i );

	virtual	void	add_option_integer( int  short_option ,
					    const std::string &  long_option ,
					    int *  i );

	//
	// Add Long Integer Option
	//
	virtual	void	add_short_option_long_integer( int  short_option ,
						       long *  i );

	virtual	void	add_long_option_long_integer
					( const std::string &  long_option ,
					  long *  i );

	virtual	void	add_option_long_integer
					( int  short_option ,
					  const std::string &  long_option ,
					  long *  i );

	//
	// Add Float Option
	//
	virtual	void	add_short_option_float( int  short_option ,
						double *  d );

	virtual	void	add_long_option_float
					( const std::string &  long_option ,
					  double *  d );

	virtual	void	add_option_float( int  short_option ,
					  const std::string &  long_option ,
					  double *  d );

	//
	// Add String Option
	//
	virtual	void	add_short_option_string( int  short_option ,
						 std::string *  str );

	virtual	void	add_long_option_string
					( const std::string &  long_option ,
					  std::string *  str );

	virtual	void	add_option_string( int  short_option ,
					   const std::string &  long_option ,
					   std::string *  str );
	//
	// Add Port Number Option
	//
	virtual	void	add_short_option_port_number( int  short_option ,
						      port_number_t *  p );

	virtual	void	add_long_option_port_number
					( const std::string &  long_option ,
					  port_number_t *  p );

	virtual	void	add_option_port_number
					( int  short_option ,
					  const std::string &  long_option ,
					  port_number_t *  p );
};


#endif	/* OPTION_ANALYZER_H_INCLUDED */
