extern "C"
{
	#include  <errno.h>
}
#include  "stream_socket.h"

#if defined(TIME_WITH_SYS_TIME) && defined(HAVE_SYS_TIME_H)
    #if TIME_WITH_SYS_TIME
	#include  <sys/time.h>
	#include  <time.h>
    #else
	#if HAVE_SYS_TIME_H
	    #include  <sys/time.h>
	#else
	    #include  <time.h>
	#endif
    #endif
#else
    #include  <sys/time.h>
#endif
#include  <unistd.h>
#include  <fcntl.h>
#include  <cstdio>
#include  <cstring>



// protected:
void   Stream_Socket::system_error( const std::string &  message ,
				    int  err ) const
{
	if ( error_stream != static_cast<std::ostream *>(0) )
	{
		(*error_stream) << message << ": "
				<< ::sys_errlist[err] << std::endl;
	}
}


// protected:
void   Stream_Socket::internal_error( const std::string &  message ) const
{
	if ( error_stream != static_cast<std::ostream *>(0) )
	{
		(*error_stream) << message << std::endl;
	}
}


Stream_Socket::Stream_Socket( Socket_Opened_Socket ,
			      int  sock_fd ,
			      int  sock_domain )
{
	this -> error_stream = &(std::cerr);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = sock_fd;
	this -> domain = sock_domain;
	this -> type   = SOCK_STREAM;
}

Stream_Socket::Stream_Socket( Socket_Opened_Socket ,
			      int  sock_fd ,
			      int  sock_domain ,
			      Socket_No_Error )
{
	this -> error_stream = static_cast<std::ostream *>(0);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = sock_fd;
	this -> domain = sock_domain;
	this -> type = SOCK_STREAM;
}

Stream_Socket::Stream_Socket( std::ostream &  err )
{
	this -> error_stream = &err;
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = -1;
	this -> domain = AF_INET;
	this -> type = SOCK_STREAM;
}


Stream_Socket::Stream_Socket( Socket_No_Error )
{
	this -> error_stream = static_cast<std::ostream *>(0);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = -1;
	this -> domain = AF_INET;
	this -> type = SOCK_STREAM;
}


Stream_Socket::Stream_Socket( int  sock_domain ,
			      int  sock_type ,
			      int  protocol ,
			      Socket_No_Error )
{
	this -> error_stream = static_cast<std::ostream *>(0);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = -1;
	this -> domain = AF_INET;
	this -> type = SOCK_STREAM;

	this -> socket( sock_domain , sock_type , protocol );
}


Stream_Socket::Stream_Socket( int  sock_domain ,
			      int  sock_type ,
			      int  protocol,
			      std::ostream &  err )
{
	this -> error_stream = &err;
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = -1;
	this -> domain = AF_INET;
	this -> type = SOCK_STREAM;

	this -> socket( sock_domain , sock_type , protocol );
}


Stream_Socket::~Stream_Socket()
{
	this -> close();
}


int    Stream_Socket::socket( int  sock_domain ,
			      int  sock_type ,
			      int  protocol )
{
	this -> domain = sock_domain;
	this -> type   = sock_type;

	if ( (sfd = ::socket( sock_domain , sock_type , protocol )) == -1 )
	{
		system_error( "socket" , errno );
	}

	return( sfd );
}


int    Stream_Socket::socket( Socket_Opened_Socket ,
			      int  sock_fd , int  sock_domain )
{
	this -> error_stream = &(std::cerr);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = sock_fd;
	this -> domain = sock_domain;
	this -> type   = SOCK_STREAM;

	return( sfd );
}

int    Stream_Socket::socket( Socket_Opened_Socket ,
			      int  sock_fd , int  sock_domain ,
			      Socket_No_Error )
{
	this -> error_stream = static_cast<std::ostream *>(0);
	this -> connected_flag = false;
	this -> end_of_file = false;
	this -> sfd = sock_fd;
	this -> domain = sock_domain;
	this -> type = SOCK_STREAM;

	return( sfd );
}


//
// close
//
int    Stream_Socket::close()
{
	int	ret;

	this -> end_of_file = true;
	this -> connected_flag = false;

	if ( sfd == -1 )
	{
		return( 0 );
	}

	if ( (ret = ::close( sfd )) == -1 )
	{
		system_error( "close" , errno );
	}
	else
	{
		if ( socket_file != "" )
		{
			if ( ::unlink( socket_file.c_str() ) == -1 )
			{
				system_error( "unlink" , errno );
			}
		}
	}

	sfd = -1;

	return( ret );
}


//
//  bind
//
int    Stream_Socket::bind( const struct sockaddr *  soc_addr,
			    SOCKLEN_T  soc_addr_len )
{
	int	ret;

	if ( (ret = ::bind( sfd , soc_addr , soc_addr_len )) == -1 )
	{
		system_error( "bind" , errno );
	}
	else
	{
		if ( soc_addr -> sa_family == AF_UNIX )
		{
			socket_file = ((const struct sockaddr_un *)soc_addr)
					-> sun_path;
		}
	}

	return( ret );
}


int    Stream_Socket::bind( const struct sockaddr_un *  soc_addr_un )
{
	int	ret;

	if ( (ret = ::bind( sfd , (const struct sockaddr *)soc_addr_un ,
			          sizeof(struct sockaddr_un) )) == -1 )
	{
		system_error( "bind" , errno );
	}
	else
	{
		socket_file = soc_addr_un -> sun_path;
	}

	return( ret );
}


int    Stream_Socket::bind( const struct sockaddr_in *  soc_addr_in )
{
	int	ret;

	if ( (ret = ::bind( sfd , (const struct sockaddr *)soc_addr_in ,
				  sizeof(struct sockaddr_in) )) == -1 )
	{
		system_error( "bind" , errno );
	}

	return( ret );
}


int    Stream_Socket::bind( const std::string &  path )
{
	struct sockaddr_un	addr;
	std::memset( &addr , 0 , sizeof(addr) );

	addr.sun_family = AF_UNIX;
	std::strncpy( addr.sun_path , path.c_str() ,
		      sizeof(addr.sun_path) );

	int	ret;
	if ( (ret = ::bind( sfd , (const struct sockaddr *)&addr ,
			    sizeof( struct sockaddr_un ) )) == -1 )
	{
		system_error( "bind" , errno );
	}
	else
	{
		socket_file = path;
	}

	return( ret );
}


int    Stream_Socket::bind( port_number_t  port )
{
	struct sockaddr_in	addr;
	std::memset( &addr , 0 , sizeof(addr) );

	addr.sin_family      = AF_INET;
	addr.sin_addr.s_addr = htonl( INADDR_ANY );
	addr.sin_port        = htons( port );

	int	ret;
	if ( (ret = ::bind( sfd , (const struct sockaddr *)&addr ,
				  sizeof( addr ) )) == -1 )
	{
		system_error( "bind" , errno );
	}

	return( ret );
}


//
//  connect
//
int    Stream_Socket::connect( const struct sockaddr *  soc_addr ,
			       SOCKLEN_T  soc_addr_len )
{
	int	ret;

	if ( (ret = ::connect( sfd , soc_addr , soc_addr_len )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}


int    Stream_Socket::connect( const struct sockaddr_un *  soc_addr_un )
{
	int	ret;

	if ( (ret = ::connect( sfd , (const struct sockaddr *)soc_addr_un ,
				     sizeof( struct sockaddr_un) )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}


int    Stream_Socket::connect( const struct sockaddr_in *  soc_addr_in )
{
	int	ret;

	if ( (ret = ::connect( sfd , (const struct sockaddr *)soc_addr_in ,
				     sizeof(struct sockaddr_in) )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}


int    Stream_Socket::connect( const std::string &  path )
{
	struct sockaddr_un	addr;
	std::memset( &addr , 0 , sizeof(addr) );

	addr.sun_family = AF_UNIX;
	std::strncpy( addr.sun_path , path.c_str() , sizeof(addr.sun_path) );

	int	ret;
	if ( (ret = ::connect( sfd , (const struct sockaddr *)&addr ,
			       sizeof( addr ) )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}


int    Stream_Socket::connect( ip_address_t  ip_addr ,
			       port_number_t  port )
{
	struct sockaddr_in	addr;
	std::memset( &addr , 0 , sizeof(addr) );

	addr.sin_family      = AF_INET;
	addr.sin_addr.s_addr = htonl( ip_addr );
	addr.sin_port        = htons( port );

	int	ret;
	if ( (ret = ::connect( sfd , (const struct sockaddr *)&addr ,
				sizeof( addr ) )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}


int    Stream_Socket::connect( const std::string &  host ,
			       port_number_t  port )
{
	struct hostent *	hp;
	ip_address_t		address;

	if ( (hp = ::gethostbyname( host.c_str() ))
	     == static_cast<struct hostent *>(0) )
	{
		std::perror( "gethostbyname" );
		return( -1 );
	}
	else
	{
		std::memcpy( &address , hp -> h_addr , hp -> h_length );

		address = htonl( address );

		return( this -> connect( address , port ) );
	}
}


int    Stream_Socket::connect( port_number_t  port )
{
	struct sockaddr_in	addr;
	std::memset( &addr , 0 , sizeof(addr) );

	addr.sin_family      = AF_INET;
	addr.sin_addr.s_addr = htonl( INADDR_ANY );
	addr.sin_port        = htons( port );

	int	ret;
	if ( (ret = ::connect( sfd , (const struct sockaddr *)&addr ,
			       sizeof( addr ) )) == -1 )
	{
		system_error( "connect" , errno );
	}
	else
	{
		this -> connected_flag = true;
	}

	return( ret );
}



//
// listen
//
int    Stream_Socket::listen( int  backlog )
{
	int	ret;

	if ( (ret = ::listen( sfd , backlog )) == -1 )
	{
		system_error( "listen" , errno );
	}

	return( ret );
}


//
//  accept
//
int    Stream_Socket::accept( struct sockaddr *  soc_addr ,
			      SOCKLEN_T *  soc_addr_len )
{
	int	ret;

	if ( (ret = ::accept( sfd , soc_addr , soc_addr_len ) ) == -1 )
	{
		system_error( "accept" , errno );
	}

	return( ret );
}


int    Stream_Socket::accept( struct sockaddr_un *  soc_addr_un )
{
	int		ret;
	SOCKLEN_T	size = sizeof(struct sockaddr_un);

	if ( (ret = ::accept( sfd , (struct sockaddr *)soc_addr_un ,
				    &size )) == -1 )
	{
		system_error( "accept" , errno );
	}

#if 0
	if ( size != sizeof(soc_addr_un) )
	{
		return( -1 );
	}
#endif

	return( ret );
}


int    Stream_Socket::accept( struct sockaddr_in *  soc_addr_in )
{
	int		ret;
	SOCKLEN_T	size = sizeof(struct sockaddr_in);

	if ( (ret = ::accept( sfd , (struct sockaddr *)soc_addr_in ,
				    &size )) == -1 )
	{
		system_error( "accept" , errno );
	}

#if 0
	if ( size != sizeof(soc_addr_in) )
	{
		return( -1 );
	}
#endif

	return( ret );
}


//
//  read
//
ssize_t Stream_Socket::read( void *  p ,  size_t  siz )
{
	ssize_t	ret;

	if ( (ret = ::read( sfd , p , siz )) == -1 )
	{
		system_error( "read" , errno );
	}

	if ( ret == 0 )
	{
		this -> end_of_file = true;
	}

	return( ret );
}

ssize_t Stream_Socket::read_all( std::string *  str )
{
	char	buf[ BUFSIZ /* in <cstdio> */ ];

	*str = "";

	for(;;)
	{
		ssize_t	n;

		if ( (n = this -> read( buf , sizeof(buf) )) == -1 )
		{
			return( -1 );
		}

		str -> append( buf , n );

		if ( n == 0 )
		{
			break;
		}
	}

	return( static_cast<ssize_t>( str -> size() ) );
}


//
//  write
//
ssize_t Stream_Socket::write( const void * p ,  size_t  siz )
{
	ssize_t	ret;

	if ( (ret = ::write( sfd , p , siz)) == -1 )
	{
		system_error( "write" , errno );
	}

	return( ret );
}

ssize_t Stream_Socket::write( const std::string &  str )
{
	return( this -> write( str.data() , str.size() ) );
}


//
//  write_all
//
ssize_t Stream_Socket::write_all( const void * p ,  size_t  siz )
{
	size_t	n_wrote = 0;

	while( n_wrote < siz )
	{
		int	n;

		if ( (n = ::write( sfd , p , siz)) == -1 )
		{
			system_error( "write" , errno );

			return( -1 );
		}

		n_wrote += n;
	}

	return( n_wrote );
}

ssize_t Stream_Socket::write_all( const std::string &  str )
{
	return( this -> write_all( str.data() , str.size() ) );
}


//
//  read
//
ssize_t Stream_Socket::recv( void *  p ,  size_t  siz ,  int  flags )
{
	int	ret;

	if ( (ret = ::recv( sfd , p , siz , flags )) == -1 )
	{
		system_error( "recv" , errno );
	}

	if ( ret == 0 )
	{
		this -> end_of_file = true;
	}

	return( ret );
}


//
//  send
//
ssize_t Stream_Socket::send( const void * p ,  size_t  siz ,  int  flags )
{
	int	ret;

	if ( (ret = ::send( sfd , p , siz , flags )) == -1 )
	{
		system_error( "send" , errno );
	}

	return( ret );
}


//
//  shutdown
//
int    Stream_Socket::shutdown( int  how )
{
	int	ret;

	if ( (ret = ::shutdown( sfd , how )) == -1 )
	{
		system_error( "shutdown" , errno );
	}

	return( ret );
}


//
// close on exec
//
int    Stream_Socket::set_close_on_exec()
{
	if ( ::fcntl( sfd , F_SETFD , 1 ) == -1 )
	{
		std::perror( "fcntl" );
		sfd = -1;
		this -> close();

		return( -1 );
	}

	return( 0 );
}


//
//  poll
//
int    Stream_Socket::poll( int  timeout /* msec */ ) const
{
	if ( sfd == -1 || end_of_file )
	{
		return( 0 );
	}


	fd_set		readfd;
	FD_ZERO( &readfd );
	FD_SET( sfd , &readfd );

	struct timeval	tv;
	tv.tv_sec  = timeout / 1000;
	tv.tv_usec = (timeout - timeout / 1000) * 1000;


	int	ret;

	if ( (ret = ::select( sfd + 1 ,
			      &readfd ,
			      static_cast<fd_set *>(0) ,
			      static_cast<fd_set *>(0) ,
			      &tv ) ) == -1 )
	{
		system_error( "select" , errno );

		return( 0 );
	}

	return( ret );
}


//
//  fd
//
int    Stream_Socket::fd() const
{
	return( sfd );
}


//
//  valid
//
bool   Stream_Socket::valid() const
{
	return( sfd != -1 );
}

//
//  operator bool
//
Stream_Socket::operator bool() const
{
	return( this -> valid() );
}

//
// eof
//
bool   Stream_Socket::eof() const
{
	return( this -> end_of_file || ! this -> valid() );
}

//
// connected
//
bool   Stream_Socket::connected() const
{
	return( this -> valid() && this -> connected_flag );
}
