#include  "string_extension.h"
#include  <string>
#include  <vector>
#include  <iostream>
#include  <cassert>

int    main( void )
{
	//
	// chomp() test
	//
	{
		std::string	str = "abcdefg\n";


		std::cout << "str = [" << str << "]" << std::endl;
		std::cout << "chomp(str) = ["
			  << String_Extension::chomp(str) <<  "]" << std::endl;

		assert( str == "abcdefg\n" );
		assert( String_Extension::chomp(str) == "abcdefg" );


		std::cout << "str = [" << str << "]" << std::endl;

		String_Extension::chomp(&str);
		std::cout << "chomp(&str)" << std::endl;
		std::cout << "str = [" << str << "]" << std::endl;

		assert( str == "abcdefg" );
	}

	std::cout << std::endl;
	std::cout << std::endl;


	//
	// split() test
	//
	{
		std::vector<std::string>	v;

		String_Extension::split_by_char_set( &v ,
						     "abc-def-ghi" , "-" );

		assert( v.size() == 3 );
		assert( v[0] == "abc" );
		assert( v[1] == "def" );
		assert( v[2] == "ghi" );

		for ( size_t  i = 0  ;  i < v.size()  ;  i ++ )
		{
			std::cout << "v[" << i << "] = [" << v[i] << "]"
				  << std::endl;
		}


		String_Extension::split_by_char_set
			( &v , "-abc-def-ghi-" , "-" , true );

		assert( v.size() == 3 );
		assert( v[0] == "abc" );
		assert( v[1] == "def" );
		assert( v[2] == "ghi" );
	}


	//
	// more split() test
	//
	{
		std::vector<std::string>	v;


		{
			String_Extension::split_by_char_set
				( &v , "" , "-" );

			assert( v.empty() );
		}


		{
			String_Extension::split_by_char_set
				( &v , "abc-def--" , "-" );

			assert( v.size() == 3 );
			assert( v[0] == "abc" );
			assert( v[1] == "def" );
			assert( v[2] == "" );
		}

		{
			String_Extension::split_by_char_set
				( &v , "--abc-def" , "-" );

			assert( v.size() == 3 );
			assert( v[0] == "" );
			assert( v[1] == "abc" );
			assert( v[2] == "def" );
		}

		{
			String_Extension::split_by_char_set
				( &v , "--abc-def--" , "-" );

			assert( v.size() == 4 );
			assert( v[0] == "" );
			assert( v[1] == "abc" );
			assert( v[2] == "def" );
			assert( v[3] == "" );
		}


		{
			String_Extension::split_by_char_set
				( &v , "a" , "-" );

			assert( v.size() == 1 );
			assert( v[0] == "a" );
		}

		{
			String_Extension::split_by_char_set
				( &v , "-*-a*bc*" , "-*" );

			assert( v.size() == 4 );
			assert( v[0] == "" );
			assert( v[1] == "a" );
			assert( v[2] == "bc" );
			assert( v[3] == "" );
		}
	}


	//
	// string_to_integer() test
	//
	{
		int	value = 0;
		bool	succeeded = String_Extension::string_to_integer
							    ( &value , "5" );

		assert( succeeded == true );
		assert( value == 5 );
	}


	//
	// string_to_long() test
	//
	{
		long	value = 0;
		bool	succeeded = String_Extension::string_to_long
							( &value , "5" );

		assert( succeeded == true );
		assert( value == 5 );
	}


	//
	// string_to_double() test
	//
	{
		double	value = 0;
		bool	succeeded = String_Extension::string_to_double
							( &value , "5.0" );

		assert( succeeded == true );
		assert( 5.0 - 0.01 < value && value < 5.0 + 0.01 );
	}


	//
	// cutoff_edge() test
	//
	{
		assert( String_Extension::cutoff_edge( "" , "\t" ) == "" );

		assert( String_Extension::cutoff_edge
			( "abc" , " \t" ) == "abc" );

		assert( String_Extension::cutoff_edge
			( " abc def " , " \t" ) == "abc def" );

		assert( String_Extension::cutoff_edge
			( "  abc def  " , " \t" ) == "abc def" );

		assert( String_Extension::cutoff_edge
			( " " , " \t" ) == "" );

		assert( String_Extension::cutoff_edge
			( "  " , " \t" ) == "" );

		assert( String_Extension::cutoff_edge
			( " \t " , " \t" ) == "" );

		assert( String_Extension::cutoff_edge
			( " a " , " \t" ) == "a" );

		assert( String_Extension::cutoff_edge
			( "  a  " , " \t" ) == "a" );

		assert( String_Extension::cutoff_edge
			( "a " , " \t" ) == "a" );

		assert( String_Extension::cutoff_edge
			( " a" , " \t" ) == "a" );

		assert( String_Extension::cutoff_edge
			( "abcdefg" , "d" ) == "abcdefg" );

		assert( String_Extension::cutoff_edge
			( "abcdefg" , "a" ) == "bcdefg" );

		assert( String_Extension::cutoff_edge
			( "aabcdefg" , "a" ) == "bcdefg" );

		assert( String_Extension::cutoff_edge
			( "\nabc\ndef\n" , "\n" ) == "abc\ndef" );
	}


	//
	// cutoff_edge_spaces() test
	//
	{
		assert( String_Extension::cutoff_edge_spaces( "" ) == "" );

		assert( String_Extension::cutoff_edge_spaces
			( "abc" ) == "abc" );

		assert( String_Extension::cutoff_edge_spaces
			( " abc def " ) == "abc def" );

		assert( String_Extension::cutoff_edge_spaces
			( "  abc def  " ) == "abc def" );

		assert( String_Extension::cutoff_edge_spaces
			( " " ) == "" );

		assert( String_Extension::cutoff_edge_spaces
			( "  " ) == "" );

		assert( String_Extension::cutoff_edge_spaces
			( " \t " ) == "" );

		assert( String_Extension::cutoff_edge_spaces
			( " a " ) == "a" );

		assert( String_Extension::cutoff_edge_spaces
			( "  a  " ) == "a" );

		assert( String_Extension::cutoff_edge_spaces
			( "a " ) == "a" );

		assert( String_Extension::cutoff_edge_spaces
			( " a" ) == "a" );

		assert( String_Extension::cutoff_edge_spaces
			( "\nabc\ndef\n" ) == "abc\ndef" );
	}


	std::cout << std::endl;
	std::cout << std::endl;
	std::cout << "All tests succeeded !!" << std::endl;

	return( 0 );
}
