#include  "string_extension.h"
#include  <cctype>
#include  <cstdio>
#include  <cstdlib>


namespace String_Extension
{


void    split_by_char_set( std::vector<std::string> *  result ,
			   const std::string &  str ,
			   const std::string &  char_set ,
			   bool  do_cutoff_edge )
{
	std::string	buf( str );

	if ( do_cutoff_edge )
	{
		cutoff_edge( &buf , char_set );
	}

	result -> clear();

	std::string::size_type	indx = 0;

	for(;;)
	{
		std::string::size_type	next_indx
					= buf.find_first_of( char_set , indx );

		if ( next_indx == std::string::npos )
		{
			if ( indx != 0 || buf.length() != 0 )
			{
				result -> push_back
					  ( std::string( buf , indx ) );
			}

			break;
		}
		else if ( next_indx == indx )
		{
			if ( indx == 0 )
			{
				result -> push_back( "" );
			}

			indx ++;
		}
		else
		{
			result -> push_back( std::string
					     ( buf ,
					       indx , next_indx - indx ) );
			indx = next_indx;
		}
	}
}


bool   string_to_integer( int *  result ,  const std::string &  str )
{
	// XXX
	*result = std::atoi( str.c_str() );
	return( true );
}

bool   string_to_long( long *  result ,  const std::string &  str )
{
	// XXX
	*result = std::atoi( str.c_str() );
	return( true );
}

bool   string_to_double( double *  result ,  const std::string &  str )
{
	char *	end_char;
	*result = std::strtod( str.c_str() , &end_char );

	if ( *end_char != '\0' )
	{
		return( false );
	}

	return( true );
}

void   double_to_string( std::string *  result ,  double  value )
{
	// XXX
	char	buf[256];

	std::sprintf( buf , "%f" , value );

	*result = buf;
}


std::string  chomp( const std::string &  str )
{
	std::string	chomped_string = str;

	chomp( &chomped_string );

	return( chomped_string );
}

void   chomp( std::string *  str )
{
	if ( str -> length() >= 1
	  && *(str -> rbegin()) == '\n' )
	{
		str -> erase( str -> end() - 1 );
	}
}

std::string  cutoff_edge( const std::string &  str ,
			  const std::string &  char_set )
{
	std::string	cutoff_string = str;

	cutoff_edge( &cutoff_string , char_set );

	return( cutoff_string );
}

void   cutoff_edge( std::string *  str ,
		    const std::string &  char_set )
{
	std::string::size_type	start_indx
				= str -> find_first_not_of( char_set );

	std::string::size_type	end_indx
				= str -> find_last_not_of( char_set );

	if ( start_indx == std::string::npos )
	{
		*str = "";
	}
	else
	{
		*str = str -> substr( start_indx , end_indx - start_indx + 1 );
	}
}


std::string  cutoff_edge_spaces( const std::string &  str )
{
	std::string	cutoff_string( str );

	cutoff_edge_spaces( &cutoff_string );

	return( cutoff_string );
}

void   cutoff_edge_spaces( std::string *  str )
{
	if ( str -> empty() )
	{
		return;
	}


	size_t	start;
	size_t	end;

	for ( start = 0  ;  start < str -> length()  ;  start ++ )
	{
		if ( ! isspace( (*str)[start] ) )
		{
			break;
		}
	}

	for ( end = str -> length() - 1  ;  end > start  ;  end -- )
	{
		if ( ! isspace( (*str)[end] ) )
		{
			break;
		}
	}

	*str = str -> substr( start , end - start + 1 );
}


} // end of namespace String_Extension
