#include  "soccer_option_analyzer.h"
#include  "debug_server_connection.h"
#include  <utility>

void   Soccer_Option_Analyzer::usage( std::ostream *  ostr ) const
{
	using std::endl;

	(*ostr)
	   << "Usage: " << this -> program_name() << " [option ...]" << endl
	   << "Possible options are:" << endl
	   << "      --help" << endl
	   << "      --version" << endl
	   << "  -h, --host, --server-host HOST" << endl
	   << "  -p, --port, --server-port PORT" << endl
	   << "  -d, --with-debug-server" << endl
	   << "      --without-debug-server" << endl
	   << "      --debug-host HOST" << endl
	   << "      --debug-port PORT" << endl
	   << "      --with-evolution-server" << endl
	   << "      --without-evolution-server" << endl
	   << "      --evolution-server-host HOST" << endl
	   << "      --evolution-server-port PORT" << endl
	   << "      --with-file-repository" << endl
	   << "      --without-file-repository" << endl
	   << "      --file-repository-path PATH" << endl
	   << "  -z, --compression-level LEVEL" << endl
	   << "  -t, --team-name TEAM_NAME" << endl
	   << "  -r, --reconnect PLAYER_NUMBER" << endl
	   << "  -g, --goalie" << endl
	   << "  -f, --field-player" << endl
	   << "  -c, --online-coach" << endl
	   << "  -v, --server-version MAJOR[.MINOR]" << endl
	   << "  -R, --role ROLE" << endl
	   << "  -T, --true-view" << endl
	   << "  -D, --debug" << endl;
}


//
// local class for --server-version option
//
class  Soccer_Server_Version_Option_Descriptor
	: public Option_Analyzer::Option_Descriptor
{
private:
	void	print_error( const std::string &  version_string ) const
	{
		std::cerr << "version number option"
			  << " error [" << version_string << "]" << std::endl;
	}

protected:
	std::pair<int,int> *	version;

public:
		 Soccer_Server_Version_Option_Descriptor
		   ( std::pair<int,int> *  v ) : version(v) {}
	virtual	~Soccer_Server_Version_Option_Descriptor() {}

	virtual	int	execute( const std::vector<std::string> &  arg ) const
	{
		if ( arg.size() < 1 )
		{
			return( -1 );
		}

		if ( ! version )
		{
			return( 1 );
		}

		char *	r;
		long	major_num;
		major_num = std::strtol( arg[0].c_str() , &r , 10 );

		if ( major_num == LONG_MIN || major_num == LONG_MAX
		  || major_num < INT_MIN   || major_num > INT_MAX )
		{
			this -> print_error( arg[0] );

			return( -1 );
		}

		if ( *r == '\0' )
		{
			version -> first = static_cast<int>( major_num );
			version -> second = 0;
		}
		else if ( r[0] == '.'  &&  r[1] != '\0' )
		{
			long	minor_num;
			char *	e;
			minor_num = std::strtol( r + 1 , &e , 10 );

			if ( *e != '\0'
			     || minor_num == LONG_MIN || minor_num == LONG_MAX
			     || minor_num < INT_MIN   || minor_num > INT_MAX )
			{
				this -> print_error( arg[0] );

				return( -1 );
			}

			version -> first  = static_cast<int>( major_num );
			version -> second = static_cast<int>( minor_num );
		}
		else
		{
			this -> print_error( arg[0] );

			return( -1 );
		}

		return( 1 );
	}
};


//
// local class for --compression-level
//
class  Compression_Level_Option_Descriptor
	: public Option_Analyzer::Integer_Option_Descriptor
{
public:
		 Compression_Level_Option_Descriptor( int  * level )
			 : Integer_Option_Descriptor( level ) {}

	virtual	~Compression_Level_Option_Descriptor() {}

	virtual	bool	check( std::string *  error_message ) const
	{
		if ( 0 <= *(this -> integer) && *(this -> integer) <= 9 )
		{
			return( true );
		}
		else
		{
			*error_message = "compression level must in [0,9].";

			return( false );
		}
	}
};


//
// local class for --debug option
//   Warning: this interface will be changed!!
//
class  Soccer_Debug_Option_Descriptor
	: public Option_Analyzer::Option_Descriptor
{
protected:
	Soccer_Option_Analyzer::Debug_Option *	debug;

public:
		 Soccer_Debug_Option_Descriptor
		   ( Soccer_Option_Analyzer::Debug_Option *  d ) : debug(d) {}
	virtual	~Soccer_Debug_Option_Descriptor() {}

	virtual	int	execute( const std::vector<std::string> & ) const
	{
		if ( debug )
		{
			// XXX: interface might be changed !!
			debug -> inhibit = ! (debug -> inhibit);
		}

		return( 0 );
	};
};


const port_number_t	Soccer_Option_Analyzer::RELATIVE_PORT    = 0;
const std::string	Soccer_Option_Analyzer::SERVER_HOST_STRING = "";

const port_number_t	Soccer_Option_Analyzer
				::EVOLUTION_SERVER_PORT_OFFSET = 70;


Soccer_Option_Analyzer::Soccer_Option_Analyzer
		( const std::string &  default_team_name ,
		  bool  default_goalie_flag ,
		  bool  default_coach_flag ,
		  const std::string &  default_host_name ,
		  port_number_t  default_port_number ,
		  bool  default_with_debug_server ,
		  const std::string &  default_debug_host_name ,
		  port_number_t  default_debug_port_number ,
		  bool  default_with_evolution_server ,
		  const std::string &  default_evolution_server_host_name ,
		  port_number_t  default_evolution_server_port_number ,
		  bool  default_with_file_repository ,
		  const std::string &  default_file_repository_path ,
		  int  default_server_major_version ,
		  int  default_server_minor_version )
{
	//
	// Initialize
	//
	this -> help_flag      = false;
	this -> client_version = false;

	this -> host        = default_host_name;
	this -> port        = default_port_number;

	this -> with_debug_server_flag = default_with_debug_server;
	this -> debug_host  = default_debug_host_name;
	this -> debug_port  = default_debug_port_number;

	this -> with_evolution_server_flag = default_with_evolution_server;
	this -> evolution_server_host = default_evolution_server_host_name;
	this -> evolution_server_port = default_evolution_server_port_number;

	this -> with_file_repository_flag = default_with_file_repository;
	this -> file_repository_path_string = default_file_repository_path;

	this -> team        = default_team_name;
	this -> goalie_flag = default_goalie_flag;
	this -> coach_flag  = default_coach_flag;

	this -> reconnect_player_num = -1;

	this -> server_version.first  = default_server_major_version;
	this -> server_version.second = default_server_minor_version;

	this -> compression_lv = 0;

	this -> role_name = "";

	this -> true_view_flag = false;

	this -> debug_option.inhibit = true;


	//
	// Add Options
	//
	this -> add_long_option_flag_on(      "--help"        , &help_flag );
	this -> add_long_option_flag_on(      "--version" ,
					      &client_version );

	this -> add_option_string     ( 'h' , "--host"        , &host );
	this -> add_long_option_string(       "--server-host" , &host );
	this -> add_option_port_number( 'p' , "--port"        , &port );
	this -> add_long_option_port_number(  "--server-port" , &port );


	this -> add_option_flag_on    ( 'd' , "--with-debug-server" ,
					      &with_debug_server_flag );
	this -> add_long_option_flag_off(     "--without-debug-server" ,
					      &with_debug_server_flag );
	this -> add_long_option_string     (  "--debug-host"  , &debug_host );
	this -> add_long_option_port_number(  "--debug-port"  , &debug_port );


	this -> add_long_option_flag_on(      "--with-evolution-server" ,
					      &with_evolution_server_flag );
	this -> add_long_option_flag_off(     "--without-evolution-server" ,
					      &with_evolution_server_flag );
	this -> add_long_option_string (      "--evolution-server-host" ,
					      &evolution_server_host );
	this -> add_long_option_port_number(  "--evolution-server-port" ,
					      &evolution_server_port );

	this -> add_long_option_flag_on(      "--with-file-repository" ,
					      &with_file_repository_flag );
	this -> add_long_option_flag_off(     "--without-file-repository" ,
					      &with_file_repository_flag );
	this -> add_long_option_string (      "--file-repository-path" ,
					      &file_repository_path_string );

	this -> add_option_string     ( 't' , "--team-name"   , &team );
	this -> add_option_flag_on    ( 'g' , "--goalie"      , &goalie_flag );
	this -> add_option_flag_off   ( 'f' , "--field-player", &goalie_flag );
	this -> add_option_flag_on    ( 'c' , "--online-coach", &coach_flag );

	this -> add_option_integer    ( 'r' , "--reconnect" ,
					      &reconnect_player_num );

	this -> add_option( 'v' , "--server-version" ,
			    new Soccer_Server_Version_Option_Descriptor
							( &server_version ) );
	this -> add_option
		( 'z' , "--compression-level" ,
		  new Compression_Level_Option_Descriptor( &compression_lv ) );

	this -> add_option_string( 'R' , "--role" , &role_name );

	this -> add_option_flag_on( 'T' , "--true-view", &true_view_flag );

	this -> add_option
		  ( 'D' , "--debug" ,
		    new Soccer_Debug_Option_Descriptor( &debug_option ) );
}

Soccer_Option_Analyzer::~Soccer_Option_Analyzer()
{
}

bool   Soccer_Option_Analyzer::help() const
{
	return( this -> help_flag );
}

bool   Soccer_Option_Analyzer::version() const
{
	return( this -> client_version );
}

std::ostream &  Soccer_Option_Analyzer::version_ostream()
{
	return( std::cout );
}

std::string  Soccer_Option_Analyzer::hostname() const
{
	return( this -> host );
}

port_number_t  Soccer_Option_Analyzer::port_number() const
{
	return( this -> port );
}


bool   Soccer_Option_Analyzer::with_debug_server() const
{
	return( this -> with_debug_server_flag );
}

std::string  Soccer_Option_Analyzer::debug_hostname() const
{
	if ( this -> debug_host == SERVER_HOST_STRING )
	{
		return( this -> hostname() );
	}
	else
	{
		return( this -> debug_host );
	}
}

port_number_t  Soccer_Option_Analyzer::debug_port_number() const
{
	if ( this -> debug_port == RELATIVE_PORT )
	{
		return( this -> port_number()
			  + Debug_Server_Connection::DEFAULT_PORT_OFFSET );
	}
	else
	{
		return( this -> debug_port );
	}
}


bool   Soccer_Option_Analyzer::with_evolution_server() const
{
	return( this -> with_evolution_server_flag );
}

std::string  Soccer_Option_Analyzer::evolution_server_hostname() const
{
	if ( this -> evolution_server_host == SERVER_HOST_STRING )
	{
		return( this -> hostname() );
	}
	else
	{
		return( this -> evolution_server_host );
	}
}

port_number_t  Soccer_Option_Analyzer::evolution_server_port_number() const
{
	if ( this -> evolution_server_port == RELATIVE_PORT )
	{
		return( this -> port_number() + EVOLUTION_SERVER_PORT_OFFSET );
	}
	else
	{
		return( this -> evolution_server_port );
	}
}


bool   Soccer_Option_Analyzer::with_file_repository() const
{
	return( this -> with_file_repository_flag );
}

std::string  Soccer_Option_Analyzer::file_repository_path() const
{
	return( this -> file_repository_path_string );
}


std::string  Soccer_Option_Analyzer::team_name() const
{
	return( this -> team );
}

bool   Soccer_Option_Analyzer::goalie() const
{
	return( this -> goalie_flag );
}

bool   Soccer_Option_Analyzer::coach() const
{
	return( this -> coach_flag );
}

bool   Soccer_Option_Analyzer::reconnect() const
{
	return( this -> reconnect_player_num != -1 );
}

int    Soccer_Option_Analyzer::reconnect_player_number() const
{
	return( this -> reconnect_player_num );
}

int   Soccer_Option_Analyzer::server_major_version() const
{
	return( server_version.first );
}

int   Soccer_Option_Analyzer::server_minor_version() const
{
	return( server_version.second );
}

int    Soccer_Option_Analyzer::compression_level() const
{
	return( this -> compression_lv );
}

std::string  Soccer_Option_Analyzer::role() const
{
	return( this -> role_name );
}

bool  Soccer_Option_Analyzer::true_view() const
{
	return( this -> true_view_flag );
}

Soccer_Option_Analyzer::Debug_Option  Soccer_Option_Analyzer::debug() const
{
	return( debug_option );
}
