#ifndef	   SOCCER_OPTION_ANALYZER_H_INCLUDED
#define	   SOCCER_OPTION_ANALYZER_H_INCLUDED

#include  <string>
#include  <utility>
#include  <iostream>
#include  "option_analyzer.h"
#include  "ip_address.h"
#include  "sserver_param.h"
#include  "sserver_player_connection.h"

class  Soccer_Option_Analyzer : public Option_Analyzer
{
public:
	static	const port_number_t	RELATIVE_PORT;
	static	const std::string	SERVER_HOST_STRING;

	static	const port_number_t	EVOLUTION_SERVER_PORT_OFFSET;

public:
	struct  Debug_Option
	{
		bool	inhibit;

		Debug_Option() : inhibit( true ) {}
	};

private:
	bool			help_flag;
	bool			client_version;

	std::string		host;
	port_number_t		port;
	bool			with_debug_server_flag;
	std::string		debug_host;
	port_number_t		debug_port;
	bool			with_evolution_server_flag;
	std::string		evolution_server_host;
	port_number_t		evolution_server_port;
	bool			with_file_repository_flag;
	std::string		file_repository_path_string;
	std::string		team;
	bool			goalie_flag;
	bool			coach_flag;
	int			reconnect_player_num;
	std::pair<int,int>	server_version;
	int			compression_lv;
	std::string		role_name;
	bool			true_view_flag;
	Debug_Option		debug_option;

public:
	 Soccer_Option_Analyzer
		    ( const std::string &  default_team_name ,
		      bool  default_goalie_flag = false ,
		      bool  default_coach_flag = false ,
		      const std::string &  default_host_name = "localhost" ,
		      port_number_t  default_port_number
					   = SServer_Param::DEFAULT_PORT ,
		      bool  default_with_debug_server = false ,
		      const std::string &  default_debug_host_name
					   = SERVER_HOST_STRING ,
		      port_number_t  default_debug_port_number
					   = RELATIVE_PORT ,
		      bool  default_with_evolution_server = false ,
		      const std::string &  default_evolution_server_host_name
					   = SERVER_HOST_STRING ,
		      port_number_t  default_evolution_server_port_number
					   = RELATIVE_PORT ,
		      bool  default_with_file_repository = false ,
		      const std::string &  default_file_repository_path = "" ,
		      int  default_server_major_version
					   = SServer_Player_Connection
					     ::SSERVER_DEFAULT_MAJOR_VERSION ,
		      int  default_server_minor_version
					   = SServer_Player_Connection
					     ::SSERVER_DEFAULT_MINOR_VERSION );

	virtual	~Soccer_Option_Analyzer();

	virtual	void	usage( std::ostream *  ostr = &std::cerr ) const;

	virtual	bool			help() const;

	virtual	bool			version() const;
	virtual	std::ostream &		version_ostream();

	virtual	std::string		hostname() const;
	virtual	port_number_t		port_number() const;

	virtual	bool			with_debug_server() const;
	virtual	std::string		debug_hostname() const;
	virtual	port_number_t		debug_port_number() const;

	virtual	bool			with_evolution_server() const;
	virtual	std::string		evolution_server_hostname() const;
	virtual	port_number_t		evolution_server_port_number() const;

	virtual	bool			with_file_repository() const;
	virtual	std::string		file_repository_path() const;

	virtual	std::string		team_name() const;
	virtual	bool			goalie() const;
	virtual	bool			coach() const;
	virtual	bool			reconnect() const;
	virtual	int			reconnect_player_number() const;
	virtual	int			server_major_version() const;
	virtual	int			server_minor_version() const;
	virtual	int			compression_level() const;

	virtual	std::string		role() const;
	virtual	bool			true_view() const;
	virtual	Debug_Option		debug() const;
};


#endif	/* SOCCER_OPTION_ANALYZER_H_INCLUDED */
