#ifndef	   S_BASIC_H_INCLUDED
#define	   S_BASIC_H_INCLUDED

#include  "d2_vector.h"
#include  <string>

static	const int	MAX_PLAYER = 11;	// max player in a team

//
//  Side
//

//
//    S_Side
//
class  S_Side
{
public:
	typedef  enum{ Unknown , Our_Side , Opponent_Side }  Side;

protected:
	S_Side::Side	side;

public:
	S_Side()
		: side( S_Side::Unknown )
		{
		}

	S_Side( S_Side::Side  s )
		: side( s )
		{
		}

	S_Side &	operator = ( S_Side::Side  s )
		{
			this -> side = s;
			return( *this );
		}

	bool	operator == ( const S_Side::Side &  s ) const
		{
			return( this -> side == s );
		}

	bool	operator != ( const S_Side::Side &  s ) const
		{
			return( this -> side != s );
		}

	operator S_Side::Side () const
		{
			return( this -> side );
		}

	S_Side	reverse() const
		{
			switch( this -> side )
			{
			case Our_Side:
				return( Opponent_Side );
				break;

			case Opponent_Side:
				return( Our_Side );
				break;

			case Unknown:
			default:
				return( Unknown );
				break;
			}
		}

	operator std::string() const
		{
			switch( this -> side )
			{
			case Our_Side:
				return( "Our_Side" );
				break;

			case Opponent_Side:
				return( "Opponent_Side" );
				break;

			case Unknown:
			default:
				return( "Unknown" );
				break;
			}
		}
};


//
//    S_Side_LR
//
class  S_Side_LR
{
public:
	typedef  enum{ Unknown , Left_Side , Right_Side }  Side;

protected:
	S_Side_LR::Side	side;

public:
	S_Side_LR()
		: side( S_Side_LR::Unknown )
		{
		}

	S_Side_LR( S_Side_LR::Side  s )
		: side( s )
		{
		}

	S_Side_LR &  operator = ( const S_Side_LR::Side &  s )
		{
			this -> side = s;
			return( *this );
		}

	bool	operator == ( const S_Side_LR::Side &  s ) const
		{
			return( this -> side == s );
		}

	bool	operator != ( const S_Side_LR::Side &  s ) const
		{
			return( this -> side != s );
		}

	operator S_Side_LR::Side () const
		{
			return( this -> side );
		}

	S_Side_LR	reverse() const
		{
			switch( this -> side )
			{
			case Left_Side:
				return( Right_Side );
				break;

			case Right_Side:
				return( Left_Side );
				break;

			case Unknown:
			default:
				return( Unknown );
				break;
			}
		}

	operator std::string() const
		{
			switch( this -> side )
			{
			case Left_Side:
				return( "Left_Side" );
				break;

			case Right_Side:
				return( "Right_Side" );
				break;

			case Unknown:
			default:
				return( "Unknown" );
				break;
			}
		}
};




//
// View mode
//

// Observational Angle of View
//     VA_Narrow: 45 degree.
//     VA_Normal: 90 degree.
//     VA_Wide:  180 degree.
//
class  View_Width
{
public:
	enum  Width { Unknown , Narrow , Normal , Wide };

protected:
	View_Width::Width	wid;

public:
	View_Width()
		: wid( View_Width::Unknown )
		{
		}

	View_Width( const View_Width::Width &  w )
		: wid( w )
		{
		}

	View_Width &  operator = ( const View_Width::Width &  w )
		{
			this -> wid = w;
			return( *this );
		}

	bool	operator == ( const View_Width::Width &  w ) const
		{
			return( this -> wid == w );
		}

	bool	operator != ( const View_Width::Width &  w ) const
		{
			return( this -> wid != w );
		}

	operator View_Width::Width () const
		{
			return( this -> wid );
		}

	Angle	angle() const
		{
			switch( wid )
			{
			case Narrow:
				return( Degree( 45.0 ) );

			case Normal:
				return( Degree( 90.0 ) );

			case Wide:
				return( Degree( 180.0 ) );

			default: // error
				return( Degree( 0.0 ) );
			}
		}

	Angle	half_angle() const
		{
			return( this -> angle() / 2.0 );
		}
};


// Quality of View
//     VQ_High: distance and direction.
//     VQ_Low:  direction only.
class  View_Quality
{
public:
	typedef enum{ Unknown , High , Low } Quality;

protected:
	View_Quality::Quality	qual;

public:
	View_Quality()
		: qual( View_Quality::Unknown )
		{
		}

	View_Quality( View_Quality::Quality  q )
		: qual( q )
		{
		}

	View_Quality &  operator = ( const View_Quality::Quality &  q )
		{
			this -> qual = q;
			return( *this );
		}

	bool	operator == ( const View_Quality::Quality &  q ) const
		{
			return( this -> qual == q );
		}

	bool	operator != ( const View_Quality::Quality &  q ) const
		{
			return( this -> qual != q );
		}

	operator View_Quality::Quality () const
		{
			return( this -> qual );
		}
};




//
//  SObject_Identifier
//
struct  SObject_Ball_Identifier
{
};

struct  SObject_Player_Identifier
{
	S_Side	side;
	int	player_number;

public:
	SObject_Player_Identifier( const S_Side &  s = S_Side::Unknown ,
				   int  n = -1 )
		: side( s ) , player_number( n ) {}

	bool	player_number_valid() const
	{
		return( 1 <= player_number  &&  player_number <= MAX_PLAYER );
	}

	bool	operator == ( const SObject_Player_Identifier &  id ) const
	{
		return(    (this -> side == id.side)
			&& (this -> player_number == id.player_number ) );
	}

	bool	operator != ( const SObject_Player_Identifier &  id ) const
	{
		return( ! (*this == id) );
	}
};

struct  SObject_Field_Marker_Identifier
{
	bool		marker_flag;
	bool		location_known;
	D2_Vector	location;
};

struct  SObject_Line_Identifier
{
	typedef  enum { Opponent_Goal_Line , Left_Wing_Line ,
			Our_Goal_Line      , Right_Wing_Line }  Which_Line;

	Which_Line	which_line;
};




//
// SServer_Time
//
struct  SServer_Time
{
private:
	long	main_step_count;
	long	sub_step_count;

public:
	SServer_Time()
		: main_step_count( 0 ) , sub_step_count( 0 ) {}

	SServer_Time( long  n )
		: main_step_count( n ) , sub_step_count( 0 ) {}

	void	set( long  main_st ,  long  sub_st = 0 )
		{
			this -> main_step_count = main_st;
			this -> sub_step_count  = sub_st;
		}

	long	main_step() const
		{
			return( this -> main_step_count );
		}

	long	sub_step() const
		{
			return( this -> sub_step_count );
		}

	bool	operator == ( const SServer_Time &  t ) const
		{
			return( this -> main_step_count == t.main_step_count
			     && this -> sub_step_count  == t.sub_step_count );
		}

	bool	operator != ( const SServer_Time &  t ) const
		{
			return( ! ((*this) == t) );
		}
};




//
// Play_Mode
//
#include  <string>
#include  <iosfwd>

class  Play_Mode
{
public:
	enum Mode { Unknown ,

		    Before_Kick_Off ,
		    Play_On ,
		    Time_Over ,

		    Our_Kick_Off ,
		    Our_Kick_In ,
		    Our_Free_Kick ,
		    Our_Indirect_Free_Kick ,
		    Our_Corner_Kick ,
		    Our_Goal_Kick ,
		    Our_Goal ,
		    Our_Catch ,

		    Opponent_Kick_Off ,
		    Opponent_Kick_In ,
		    Opponent_Free_Kick ,
		    Opponent_Indirect_Free_Kick ,
		    Opponent_Corner_Kick ,
		    Opponent_Goal_Kick ,
		    Opponent_Goal ,
		    Opponent_Catch
	};

	operator std::string() const
		{
			const char *	play_mode_str[] = {
				"Unknown" ,

				"Before Kick Off" ,
				"Play On" ,
				"Time Over" ,

				"Our Kick Off" ,
				"Our Kick In" ,
				"Our Free Kick" ,
				"Our Indirect Free Kick" ,
				"Our Corner Kick" ,
				"Our Goal Kick" ,
				"Our Goal" ,
				"Our Catch" ,

				"Opponent Kick Off" ,
				"Opponent Kick In" ,
				"Opponent Free Kick" ,
				"Opponent Indirect Free Kick" ,
				"Opponent Corner Kick" ,
				"Opponent Goal Kick" ,
				"Opponent Goal" ,
				"Opponent Catch"
			};

			return( static_cast<std::string>
				( play_mode_str[ this -> mode ] ) );
		}

private:
	Play_Mode::Mode		mode;

public:
	Play_Mode()
		: mode( Play_Mode::Unknown )
		{
		}

	Play_Mode( const Play_Mode::Mode &  m )
		: mode( m )
		{
		}

	operator Play_Mode::Mode() const
		{
			return( this -> mode );
		}

	Play_Mode	operator = ( const Play_Mode::Mode &  m )
		{
			this -> mode = m;
			return( *this );
		}

	bool	operator == ( const Play_Mode::Mode &  mode ) const
		{
			return( this -> mode == mode );
		}

	bool	operator != ( const Play_Mode::Mode &  mode ) const
		{
			return( this -> mode != mode );
		}

#if 0
	// XXX
	bool	we_must_kick() const
		{
			return( this -> mode == Our_Kick_Off
			     || this -> mode == Our_Kick_In
			     || this -> mode == Our_Free_Kick
			     || this -> mode == Our_Corner_Kick
			     || this -> mode == Our_Goal_Kick );
		}

	// XXX
	bool	we_may_kick() const
		{
			return( this -> mode == Play_On
			     || this -> we_must_kick() );
		}

	// XXX
	bool	can_move() const
		{
			return( this -> mode == Before_Kick_Off );
		}
#endif
};

#include  <iostream>

inline
std::ostream &  operator << ( std::ostream &  ostr ,  const Play_Mode &  mode )
{
	return( ostr << static_cast<std::string>(mode) );
}


//
// Judgement_Type
//
#include  <string>
#include  <iosfwd>

class  Judgement_Type
{
public:
	enum Type {
		Unknown ,
		Goal , Foul , Offside , Drop_Ball ,
		Goalie_Catch_Ball ,
		Back_Pass , Free_Kick_Fault ,
		Catch_Fault ,
		Half_Time , Time_Up , Time_Up_Without_A_Team ,
		Time_Extended ,
		One_of_Play_Modes
	};

	operator std::string() const
		{
			const char *	judgement_type_str[] = {
				"Unknown" ,
				"Goal" , "Foul" , "Offside" , "Drop_Ball" ,
				"Goalie_Catch_Ball" ,
				"Back_Pass" , "Free_Kick_Fault" ,
				"Catch_Fault" ,
				"Half_Time" ,
				"Time_Up" , "Time_Up_Without_A_Team" ,
				"Time_Extended" ,
				"One_of_Play_Modes"
			};

			return( static_cast<std::string>
				( judgement_type_str[ this -> type ] ) );
		}
protected:
	Judgement_Type::Type	type;

public:
	Judgement_Type()
		: type( Judgement_Type::Unknown )
		{
		}

	Judgement_Type( const Judgement_Type::Type &  t )
		: type( t )
		{
		}

	operator Judgement_Type::Type() const
		{
			return( this -> type );
		}

	Judgement_Type	operator = ( const Judgement_Type::Type &  t )
		{
			this -> type = t;
			return( *this );
		}

	bool	operator == ( const Judgement_Type::Type &  type ) const
		{
			return( this -> type == type );
		}

	bool	operator != ( const Judgement_Type::Type &  type ) const
		{
			return( this -> type != type );
		}
};

inline
std::ostream&  operator << ( std::ostream &  ostr ,
			     const Judgement_Type &  type )
{
	return( ostr << static_cast<std::string>(type) );
}


#endif	/* S_BASIC_H_INCLUDED */
