#include  "client_debug_view.h"
#include  "debug_server_connection.h"
#include  "field_recog_interface.h"
#include  "s_basic.h"
#include  "d2_region.h"
#include  "compat_stringstream.h"
#include  <iostream>
#include  <cstdio>
#include  <cfloat>
#include  <cstring>
#include  <stdexcept>
#include  <cassert>

using namespace std;

Client_Debug_View::Client_Debug_View( const SServer_Param *  param )
	: param( param ) ,
	  b_comment() ,
	  t_comment( MAX_PLAYER ) , o_comment( MAX_PLAYER ) ,
	  ut_comment() , uo_comment() , u_comment() ,
	  have_target_teammate_flag( false ) , target_teammate( -1 ) ,
	  have_target_point_flag( false ) , target_point() ,
	  have_message_flag( false ) , message() ,
	  debug_lines()
{
}

Client_Debug_View::~Client_Debug_View()
{
}

void   Client_Debug_View::set_field_snapshot
       ( const ref_count_ptr<const Field_Recog_Abstract
					::Field_Recog_Snapshot> &  f )
{
	field = f;
}

ref_count_ptr<const Field_Recog_Abstract::Field_Recog_Snapshot>
	Client_Debug_View::field_snapshot() const
{
	return( field );
}

const std::string &  Client_Debug_View::ball_comment() const
{
	return( b_comment );
}

const std::string &  Client_Debug_View::teammate_comment( int  num ) const
{
	assert( 1 <= num && num <= MAX_PLAYER );
	return( t_comment[num -1] );
}

const std::string &  Client_Debug_View::opponent_comment( int  num ) const
{
	assert( 1 <= num && num <= MAX_PLAYER );
	return( o_comment[num -1] );
}

const std::string &  Client_Debug_View::unknown_teammate_comment
							( size_t  indx ) const
{
	assert( indx < ut_comment.size() );
	return( ut_comment[indx] );
}

const std::string &  Client_Debug_View::unknown_opponent_comment
							( size_t  indx ) const
{
	assert( indx < uo_comment.size() );
	return( uo_comment[indx] );
}

const std::string &  Client_Debug_View::unknown_player_comment
							( size_t  indx ) const
{
	assert( indx < u_comment.size() );

	return( u_comment[indx] );
}


bool   Client_Debug_View::encode( std::string *  buf ) const
{
	return( this -> encode_version_2( buf ) );
}

bool   Client_Debug_View::encode_version_2( std::string *  buf ) const
try
{
	if ( ! field )
	{
		*buf = "";

		return( false );
	}

#ifdef SHORT_DEBUG_MESSAGE
	if ( ! (this -> have_target_teammate()
		|| this -> have_target_point()
		|| this -> have_message()
		|| this -> debug_lines.size() != 0) )
	{
		return( false );
	}
#endif

	compat_ostringstream	str;

#if 0
	str.precision( DBL_DIG );
#endif

#ifdef SHORT_DEBUG_MESSAGE
	str.precision( 4 );
#endif

	Field_Recog_Interface	f( field );

	std::string	side;
	switch( f.our_team().side_lr() )
	{
	case S_Side_LR::Left_Side:
		side = "l";
		break;

	case S_Side_LR::Right_Side:
		side = "r";
		break;

	case S_Side_LR::Unknown:
	default:
		*buf = "";
		return( false );
	}


	str << "((debug (format-version 2))";

	str << " (time " << f.current_time().main_step() << ")";

	str << " "
	    << "(s "
	    <<    side << " "
	    <<    f.self().player_number() << " "
	    <<    f.self().coordinate().x() << " "
	    <<    f.self().coordinate().y() << " "
	    <<    f.self().velocity().x() << " "
	    <<    f.self().velocity().y() << " "
	    <<    f.self().body_angle().degree() << " "
	    <<    f.self().face_angle().degree();

	if ( ! f.self().coordinate().accuracy_check( 0 ) )
	{
		str << " (c \"" << f.self().coordinate().accuracy_level()
		    <<      "\")";
	}

	str << ")";


	if ( f.ball().coordinate().have_info() )
	{
		str << " "
		    << "(b "
		    <<   f.ball().coordinate().x() << " "
		    <<   f.ball().coordinate().y() << " "
		    <<   f.ball().velocity().x() << " "
		    <<   f.ball().velocity().y() << " "
		    <<   "(c \""
		    <<      f.ball().coordinate().accuracy_level() << "\")"
		    << ")";
	}


#ifndef SHORT_DEBUG_MESSAGE
	str.precision( 4 );

	Player_Set	s = f.player_set
				( new All_Player_Except_Self_Predicate );

	for ( Player_Set::iterator  it = s.begin()  ;
	      it != s.end()  ;  it ++ )
	{
		if ( it -> coordinate().have_info() )
		{
			str << " "
			    << "(";

			if ( it -> teammate() )
			{
				if ( it -> player_number_known() )
				{
					str << "t " << it -> player_number();
				}
				else
				{
					str << "ut";
				}
			}
			else if ( it -> opponent() )
			{
				if ( it -> player_number_known() )
				{
					str << "o " << it -> player_number();
				}
				else
				{
					str << "uo";
				}
			}
			else
			{
				str << "u";
			}

			str << " "
			    << it -> coordinate().x() << " "
			    << it -> coordinate().y() << " "
			    << "(c \"" << it -> coordinate()
						   .accuracy_level() << "\")";

			if ( it -> body_angle().accuracy_check() )
			{
				str << " "
				    << "(bd "
				    <<   it -> body_angle().degree()
				    << ")";
			}

			str << ")";
		}
	}
#else
	str.precision( 3 );

	if ( this -> have_target_teammate()
	  && f.teammate( this -> get_target_teammate() )
						.coordinate().have_info() )
	{
		int	i = this -> get_target_teammate();

		str << " "
		    << "(t "
		    <<   i << " "
		    <<	 f.teammate(i).coordinate().x() << " "
		    <<   f.teammate(i).coordinate().y() << " "
		    <<   "(c \"" << f.teammate(i).coordinate()
						.accuracy_level() << "\")";

		if ( f.teammate(i).body_angle().accuracy_check() )
		{
			str << " "
			    << "(bd "
			    <<   f.teammate(i).body_angle().degree()
			    << ")";
		}

		str << ")";
	}
#endif


#ifdef SHORT_DEBUG_MESSAGE
	str.precision( 3 );
#else
	str.precision( 6 );
#endif

#if 0
	str << " "
	    << "(defence-line " << f.our_team().defence_line() << ")";

	str << " "
	    << "(offence-line " << f.our_team().offence_line() << ")";
#endif

	// Warning: This protocol is only in transition period.
	//          Don't use (or use carefully) this protocol !!
	if ( this -> have_target_teammate() )
	{
		str << " (target-teammate "
		    <<    this -> get_target_teammate() << ")";
	}

	// Warning: This protocol is only in transition period.
	//          Don't use (or use carefully) this protocol !!
	if ( this -> have_target_point() )
	{
		str << " (target-point "
		    <<    this -> get_target_point().x() << " "
		    <<    this -> get_target_point().y() << ")";
	}

	// Warning: This protocol is only in transition period.
	//          Don't use (or use carefully) this protocol !!
	if ( this -> have_message() )
	{
		str << " (message \""
		    <<    this -> get_message() << "\")";
	}


	for ( size_t  i = 0  ;  i < this -> debug_lines.size()  ;  i ++ )
	{
		str << " (line "
		    <<    this -> debug_lines[i].first.x() << " "
		    <<    this -> debug_lines[i].first.y() << " "
		    <<    this -> debug_lines[i].second.x() << " "
		    <<    this -> debug_lines[i].second.y() << ")";
	}

	str << ")";

	*buf = str.str();

	return( true );
}
catch( const std::bad_alloc & )
{
	cerr << "Not enough memory for send debug info !!" << endl;

	*buf = "";

	return( false );
}




bool   Client_Debug_View::decode( const std::string &  buf )
{
	int	version = 0;

	if ( std::sscanf( buf.c_str() ,
			  " ( ( debug ( format-version %d ) )" ,
			  &version ) == 1 )
	{
		switch( version )
		{
		case 1:
			std::cerr
			 << "debug message version 1 is not supported anymore."
			 << std::endl;
			return( false );
			break;

		case 2:
			return( decode_version_2( buf ) );
			break;

		default:
			return( false );
		}
	}
	else
	{
		return( false );
	}
}

bool   Client_Debug_View::decode_version_2( const std::string &  buf )
{
	ref_count_ptr<Field_Recog_Abstract::Field_Recog_Snapshot>	f
		= new Field_Recog_Abstract::Field_Recog_Snapshot( param );

	int	read_offset = 0;

	if ( std::sscanf( buf.c_str() ,
			  " ( ( debug ( format-version 2 ) ) %n" ,
			  &read_offset ) != 0 )
	{
		cerr << "bad format-version" << endl;

		return( false );
	}


	for(;;)
	{
		// teammate player
		{
			int	n_read = 0;
			int	uniform_number = -1;
			double	pos_x;
			double	pos_y;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( t %d %lf %lf %n" ,
					  &uniform_number ,
					  &pos_x , &pos_y ,
					  &n_read ) == 3
			     && (n_read != 0) )
			{
				read_offset += n_read;

				// uniform_number
				if ( uniform_number < 1
				     || uniform_number > MAX_PLAYER )
				{
					cerr << "bad uniform-number"
					     << endl;
					return( false );
				}

				// coordinate
				f -> teammate[uniform_number - 1].coordinate
					 .set( D2_Region::universal_region() ,
					       D2_Vector( pos_x , pos_y ) );

				f -> teammate[uniform_number - 1]
					 .coordinate_accuracy = 0;

				for(;;)
				{
				    // body-direction
				    {
					int	n = 0;
					double	body_direction = 0.0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ( bd %lf ) %n" ,
					       &body_direction , &n ) == 1
					     && (n != 0) )
					{
					    read_offset += n;

					    f -> teammate[uniform_number - 1]
						.body_angle
						.set( Degree(body_direction) ,
						      Degree(body_direction) ,
						      Degree(body_direction) );

					    continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	buffer[256 + 1];

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       buffer , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> t_comment
							  [uniform_number - 1]
							.assign( buffer );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}


		// opponent player
		{
			int	n_read = 0;
			int	uniform_number = -1;
			double	pos_x;
			double	pos_y;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( o %d %lf %lf %n" ,
					  &uniform_number ,
					  &pos_x , &pos_y ,
					  &n_read ) == 3
			     && (n_read != 0) )
			{
				read_offset += n_read;

				// uniform_number
				if ( uniform_number < 1
				     || uniform_number > MAX_PLAYER )
				{
					cerr << "bad uniform-number"
					     << endl;
					return( false );
				}

				// coordinate
				f -> opponent[uniform_number - 1].coordinate
					 .set( D2_Region::universal_region() ,
					       D2_Vector( pos_x , pos_y ) );

				f -> opponent[uniform_number - 1]
					 .coordinate_accuracy = 0;

				for(;;)
				{
				    // body-direction
				    {
					int	n = 0;
					double	body_direction = 0.0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ( bd %lf ) %n" ,
					       &body_direction , &n ) == 1
					     && (n != 0) )
					{
					    read_offset += n;

					    f -> opponent[uniform_number - 1]
						.body_angle
						.set( Degree(body_direction) ,
						      Degree(body_direction) ,
						      Degree(body_direction) );

					    continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	buffer[256 + 1];

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       buffer , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> o_comment
							  [uniform_number - 1]
							.assign( buffer );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}


		// unknown teammate player
		{
			int	n_read = 0;
			double	pos_x;
			double	pos_y;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( ut %lf %lf %n" ,
					  &pos_x , &pos_y ,
					  &n_read ) == 2
			     && (n_read != 0) )
			{
				read_offset += n_read;

				Field_Recog_Abstract::Player_Info	pl;

				// coordinate
				pl.coordinate.set
					   ( D2_Region::universal_region() ,
					     D2_Vector( pos_x , pos_y ) );

				pl.coordinate_accuracy = 0;

				f -> unknown_teammate.push_back( pl );
				this -> ut_comment.push_back( "" );


				for(;;)
				{
				    // body-direction
				    {
					int	n = 0;
					double	body_direction = 0.0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ( bd %lf ) %n" ,
					       &body_direction , &n ) == 1
					     && (n != 0) )
					{
					    read_offset += n;

					    f -> unknown_teammate.rbegin()
					      -> body_angle
						.set( Degree(body_direction) ,
						      Degree(body_direction) ,
						      Degree(body_direction) );

					    continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	buffer[256 + 1];

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       buffer , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> ut_comment.rbegin()
							-> assign( buffer );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}

		// unknown opponent player
		{
			int	n_read = 0;
			double	pos_x;
			double	pos_y;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( uo %lf %lf %n" ,
					  &pos_x , &pos_y ,
					  &n_read ) == 2
			     && (n_read != 0) )
			{
				read_offset += n_read;

				Field_Recog_Abstract::Player_Info	pl;

				// coordinate
				pl.coordinate.set
					   ( D2_Region::universal_region() ,
					     D2_Vector( pos_x , pos_y ) );

				pl.coordinate_accuracy = 0;

				f -> unknown_opponent.push_back( pl );
				this -> uo_comment.push_back( "" );


				for(;;)
				{
				    // body-direction
				    {
					int	n = 0;
					double	body_direction = 0.0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ( bd %lf ) %n" ,
					       &body_direction , &n ) == 1
					     && (n != 0) )
					{
					    read_offset += n;

					    f -> unknown_opponent.rbegin()
					      -> body_angle
						.set( Degree(body_direction) ,
						      Degree(body_direction) ,
						      Degree(body_direction) );

					    continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	buffer[256 + 1];

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       buffer , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> uo_comment.rbegin()
							-> assign( buffer );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}

		// unknown player
		{
			int	n_read = 0;
			double	pos_x;
			double	pos_y;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( u %lf %lf %n" ,
					  &pos_x , &pos_y , &n_read ) == 2
			     && (n_read != 0) )
			{
				read_offset += n_read;

				Field_Recog_Abstract::Player_Info	pl;

				pl.coordinate
					.set( D2_Region::universal_region() ,
					      D2_Vector( pos_x , pos_y ) );

				f -> unknown_player.push_back( pl );
				this -> u_comment.push_back( "" );

				for(;;)
				{
				    // body-direction
				    {
					int	n = 0;
					double	body_direction = 0.0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ( bd %lf ) %n" ,
					       &body_direction , &n ) == 1
					     && (n != 0) )
					{
					    read_offset += n;

					    f -> unknown_player.rbegin()
					      -> body_angle
						.set( Degree(body_direction) ,
						      Degree(body_direction) ,
						      Degree(body_direction) );

					    continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	buffer[256 + 1];

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       buffer , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> u_comment.rbegin()
							-> assign( buffer );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}


		// time
		{
			long	current_time = -1;
			int	n_read = 0;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ( time %ld ) %n" ,
					  &current_time , &n_read ) == 1
			     && (n_read != 0) )
			{
				read_offset += n_read;

				f -> game_state.current_time
							.set( current_time );
				continue;
			}
		}

		// ball (obsolete format)
		{
			double	pos_x;
			double	pos_y;
			double	vel_x;
			double	vel_y;
			char	comment[256 + 1] = "";
			int	n_read = 0;

			if ( std::sscanf
			     ( buf.c_str() + read_offset ,
			       " ( b %lf %lf %lf %lf \"%256[^\"]\" ) %n" ,
			       &pos_x , &pos_y , &vel_x , &vel_y , comment ,
			       &n_read ) == 5
			     && (n_read != 0) )
			{
				read_offset += n_read;

				f -> ball.coordinate
					.set( D2_Region::universal_region() ,
					      D2_Vector( pos_x , pos_y ) );
				f -> ball.coordinate_accuracy = 0;

				f -> ball.velocity
					.set( D2_Region::universal_region() ,
					      D2_Vector( vel_x , vel_y ) );
				f -> ball.velocity_accuracy = 0;

				this -> b_comment.assign( comment );

				continue;
			}
		}

		// ball
		{
			double	pos_x;
			double	pos_y;
			int	n_read = 0;

			if ( std::sscanf
			     ( buf.c_str() + read_offset ,
			       " ( b %lf %lf %n" ,
			       &pos_x , &pos_y , &n_read ) == 2
			     && (n_read != 0) )
			{
				read_offset += n_read;

				f -> ball.coordinate
					.set( D2_Region::universal_region() ,
					      D2_Vector( pos_x , pos_y ) );
				f -> ball.coordinate_accuracy = 0;

				for(;;)
				{
				    // velocity
				    {
					int	n = 0;
					double	vel_x;
					double	vel_y;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " %lf %lf %n" ,
					       &vel_x , &vel_y , &n ) == 2
					     && (n != 0) )
					{
						read_offset += n;

						f -> ball.velocity
						   .set( D2_Region
							 ::universal_region() ,
							 D2_Vector( vel_x ,
								    vel_y ) );

						f -> ball.velocity_accuracy
							= 0;

						continue;
					}
				    }

				    // comment
				    {
					int	n = 0;
					char	comment[256 + 1] = "";

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       comment , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> b_comment
							.assign( comment );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}


		// self
		{
			int	n_read = 0;
			char	side;
			int	uniform_number = -1;
			double	pos_x;
			double	pos_y;
			double	vel_x;
			double	vel_y;
			double	body_direction;
			double	face_direction;

			if ( std::sscanf
			     ( buf.c_str() + read_offset ,
			       " ( s %c %d %lf %lf %lf %lf %lf %lf %n" ,
					  &side , &uniform_number ,
					  &pos_x , &pos_y , &vel_x , &vel_y ,
					  &body_direction , &face_direction ,
					  &n_read ) == 8
			     && (n_read != 0) )
			{
				read_offset += n_read;

				// side
				if ( side == 'l' )
				{
					f -> game_state.our_side
						= S_Side_LR::Left_Side;
				}
				else if ( side == 'r' )
				{
					f -> game_state.our_side
						= S_Side_LR::Right_Side;
				}
				else
				{
					cerr << "bad team side" << endl;
					return( false );
				}


				// uniform-number
				if ( uniform_number < 1
				     || uniform_number > MAX_PLAYER )
				{
					cerr << "bad uniform-number" << endl;
					return( false );
				}

				f -> game_state.self_player_number
					= uniform_number;

				// coordinate
				f -> body.my_coordinate
					.set( D2_Region::universal_region() ,
					      D2_Vector( pos_x , pos_y ) );

				if ( uniform_number < 1
				     || uniform_number > MAX_PLAYER )
				{
					cerr << "uniform-number must be "
						"specified before "
						"coordinate" << endl;

					return( false );
				}

				f -> teammate[uniform_number - 1].coordinate
					.set( D2_Region::universal_region() ,
					      D2_Vector( pos_x , pos_y ) );

				f -> teammate[uniform_number - 1]
					.coordinate_accuracy = 0;


				// body-direction
				f -> body.my_body_angle
					      .set( Degree(body_direction) ,
						    Degree(body_direction) ,
						    Degree(body_direction) );

				f -> teammate[uniform_number - 1].body_angle
					  .set( Degree(body_direction) ,
						Degree(body_direction) ,
						Degree(body_direction) );

				// neck-direction
				f -> body.neck_angle
					      .set( Degree(face_direction) ,
						    Degree(face_direction) ,
						    Degree(face_direction) );

				f -> teammate[uniform_number - 1].face_angle
					  .set( Degree(face_direction) ,
						Degree(face_direction) ,
						Degree(face_direction) );

				for(;;)
				{
				    // comment
				    {
					int	n = 0;
					char	comment[256 + 1] = "";

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       "( c \"%256[^\"]\" ) %n" ,
					       comment , &n ) == 1
					     && (n != 0) )
					{
						read_offset += n;

						this -> t_comment
							  [uniform_number - 1]
							.assign( comment );

						continue;
					}
				    }

				    // close paren
				    {
					int	n = 0;

					if ( std::sscanf
					     ( buf.c_str() + read_offset ,
					       " ) %n" , &n ) == 0
					     && (n != 0) )
					{
						read_offset += n;
						break;
					}
				    }
				}

				continue;
			}
		}

		// Warning: This protocol is only in transition period.
		//          Don't use (or use carefully) this protocol !!
		// target-player
		{
			int	n_read = 0;
			int	pl_num;

			if ( (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( target-teammate %d ) %n" ,
				&pl_num , &n_read ) == 1
			      && (n_read != 0))
			  || (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( unofficial-target-teammate %d ) %n" ,
				&pl_num , &n_read ) == 1
			      && (n_read != 0)) )
			{
				read_offset += n_read;

				this -> set_target_teammate( pl_num );

				continue;
			}
		}

		// Warning: This protocol is only in transition period.
		//          Don't use (or use carefully) this protocol !!
		// target-point
		{
			int	n_read = 0;
			double	target_x;
			double	target_y;

			if ( (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( target-point %lf %lf ) %n" ,
				&target_x , &target_y , &n_read ) == 2
			      && (n_read != 0))
			  || (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( unofficial-target-point %lf %lf ) %n" ,
				&target_x , &target_y , &n_read ) == 2
			      && (n_read != 0)) )
			{
				read_offset += n_read;

				this -> set_target_point
					( D2_Vector( target_x , target_y ) );
				continue;
			}
		}

		// Warning: This protocol is only in transition period.
		//          Don't use (or use carefully) this protocol !!
		// message
		{
			int	n_read = 0;
			char	mes[256 + 1];
			mes[sizeof(mes) - 1] = '\0';

			if ( (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( message \"%256[^\"]\" ) %n" ,
				mes , &n_read ) == 1
			      && (n_read != 0))
			  || (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( unofficial-message \"%256[^\"]\" ) %n" ,
				mes , &n_read ) == 1
			      && (n_read != 0)) )
			{
				read_offset += n_read;

				this -> set_message( mes );

				continue;
			}
		}


		// Warning: This protocol is only in transition period.
		//          Don't use (or use carefully) this protocol !!
		// line
		{
			int	n_read = 0;
			double	a_x;
			double	a_y;
			double	b_x;
			double	b_y;

			if ( (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( line %lf %lf %lf %lf) %n" ,
				&a_x , &a_y , &b_x , &b_y , &n_read ) == 4
			      && (n_read != 0))
			  || (std::sscanf
			      ( buf.c_str() + read_offset ,
				" ( unofficial-line %lf %lf %lf %lf) %n" ,
				&a_x , &a_y , &b_x , &b_y , &n_read ) == 4
			      && (n_read != 0)) )
			{
				read_offset += n_read;

				this -> add_debug_line
					( D2_Vector( a_x , a_y ) ,
					  D2_Vector( b_x , b_y ) );

				continue;
			}
		}


		// close paren
		{
			int	n_read = 0;

			if ( std::sscanf( buf.c_str() + read_offset ,
					  " ) %n" ,
					  &n_read ) == 0
			     && (n_read != 0) )
			{
				read_offset += n_read;

				break;
			}
		}

		return( false );
	}


	// if message is still remained,
	//   treat this message as a bad format one.
	if ( *(buf.c_str() + read_offset) != '\0'
	  || (   *(buf.c_str() + read_offset    ) == '\n'
	      && *(buf.c_str() + read_offset + 1) == '\0' ) )
	{
		return( false );
	}

	field = f;

	return( true );
}


// XXX
void   Client_Debug_View::set_target_teammate( int  num ) const
{
	this -> target_teammate = num;
	this -> have_target_teammate_flag = true;
}

// XXX
bool   Client_Debug_View::have_target_teammate() const
{
	return( this -> have_target_teammate_flag );
}

// XXX
int    Client_Debug_View::get_target_teammate() const
{
	return( this -> target_teammate );
}


// XXX
void   Client_Debug_View::set_target_point( const D2_Vector &  point ) const
{
	this -> target_point = point;
	this -> have_target_point_flag = true;
}

// XXX
bool   Client_Debug_View::have_target_point() const
{
	return( this -> have_target_point_flag );
}

// XXX
D2_Vector  Client_Debug_View::get_target_point() const
{
	return( this -> target_point );
}


// XXX
void   Client_Debug_View::set_message( const std::string &  mes ) const
{
	this -> message = mes;
	this -> have_message_flag = true;
}

// XXX
bool   Client_Debug_View::have_message() const
{
	return( this -> have_message_flag );
}

// XXX
std::string  Client_Debug_View::get_message() const
{
	return( this -> message );
}

// XXX
void   Client_Debug_View::add_debug_line( const D2_Vector &  a ,
					  const D2_Vector &  b )
{
	this -> debug_lines
		.push_back( std::pair<D2_Vector, D2_Vector>( a , b ) );
}

// XXX
std::vector< std::pair<D2_Vector, D2_Vector> >  Client_Debug_View
						      ::get_debug_lines() const
{
	return( this -> debug_lines );
}
