#ifndef	   FIELD_RECOG_INTERFACE_H_INCLUDED
#define	   FIELD_RECOG_INTERFACE_H_INCLUDED

// Author:		H. Shimora
// Last-Modified:	May 21 2000
// Created:		May 21 2000
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  May 21 2000    base version.
//
//

#include  "soccer_object_reference.h"
#include  "player_set.h"

#include  "field_recog_abstract.h"
#include  "s_basic.h"
#include  "sserver_param.h"
#include  "soccer_action.h"
#include  "d2_vector.h"
#include  "ref_count_ptr.h"

#include  "evolutional_repository.h"

#include  <vector>
#include  <string>
#include  <utility>
#include  <climits>


//
// Field_Recog_Interface
//
class  Field_Recog_Interface
{
private:
	// not allowed, no implementation
	Field_Recog_Interface( const Field_Recog_Interface & );
	Field_Recog_Interface &	operator = ( const Field_Recog_Interface & );

private:
	const Field_Recog_Abstract *	field;
	ref_count_ptr<const Field_Recog_Abstract::Field_Recog_Snapshot>	snap;

	struct Action_Set_Implement;
	mutable	ref_count_ptr< struct Action_Set_Implement >
							actions_implement;

	ref_count_ptr<Evolutional_Repository>	evolutional_repository;


	// XXX: don't use this !!
	mutable	bool		have_target_teammate_flag;
	mutable	int		target_teammate;

	// XXX: don't use this !!
	mutable	bool		have_target_point_flag;
	mutable	D2_Vector	target_point;

	// XXX: don't use this !!
	mutable	bool		have_message_flag;
	mutable	std::string	message;

	// XXX: don't use this !!
	mutable	std::vector< std::pair<D2_Vector, D2_Vector> >	debug_lines;

public:
	const Field_Recog_Abstract::Field_Recog_Snapshot &	snapshot()
									const;
public:
	void	update();

private:
	mutable	ref_count_ptr<Ball_Reference>	ball_reference_cache;
	mutable	std::vector<Player_Reference>	teammate_reference_cache;
	mutable	std::vector<Player_Reference>	opponent_reference_cache;
	mutable	Player_Reference *		self_reference_cache;

	void					make_cache() const;

public:
	 Field_Recog_Interface( const Field_Recog_Abstract *  field ,
				const ref_count_ptr<Evolutional_Repository> &
						      evolutional_repository );
	 Field_Recog_Interface
		 ( const ref_count_ptr<const Field_Recog_Abstract
					   ::Field_Recog_Snapshot> &  snap );
	 Field_Recog_Interface();
	~Field_Recog_Interface();

	ref_count_ptr<Soccer_Action>
		register_action
		    ( const ref_count_ptr<Soccer_Action> & ) const /* XXX */;

	ref_count_ptr<Evolutional_Repository>	evs() const /* XXX */;

	// XXX: should return ref_count_ptr<Field_Recog_Interface>
	const Field_Recog_Abstract::Debug_Info	debug() const;

	// XXX: this interface might be changed
	const SServer_Param &	sserver_param() const;

	bool			game_over() const;

	SServer_Time		current_time() const;
	Play_Mode_Reference	play_mode() const;


	const Ball_Reference &		ball() const;

	const Player_Reference &	self() const;
	const Player_Reference &	teammate( int  number ) const;
	const Player_Reference &	opponent( int  number ) const;
	Player_Reference		player( const S_Side &  side ,
						int  number ) const;

	// XXX
	size_t			n_unknown_teammate() const;
	Player_Reference	unknown_teammate( int  indx ) const;

	size_t			n_unknown_opponent() const;
	Player_Reference	unknown_opponent( int  indx ) const;

	size_t			n_unknown_player() const;
	Player_Reference	unknown_player( int  indx ) const;


	Team_Reference	our_team() const;
	Team_Reference	opponent_team() const;
	Team_Reference	team( const S_Side &  side ) const;

	// XXX: temporary interface
	bool	offside_position( const D2_Vector & ) const;

	// XXX: temporary interface
	const Player_Type_Manager &	player_type_manager() const;

	Player_Set	player_set( const Player_Predicate & ) const;

	Soccer_Composite_Command	previous_command() const;

	D2_Vector	ball_predict_point( long  t = LONG_MIN ) const;

	long		strict_ball_loose_step( long  max_step = 10 ,
						long  acc_level = - 50 ) const;

	// XXX: temporary interface
	void	ball_access_step( const Player_Set &  players ,
				  long *  self_access_step ,
				  long *  teammate_access_step ,
				  long *  opponent_access_step ,
				  long *  unknown_access_step ,
				  long  max_step = -1 ,
				  long  acc_level = -50 ) const;

	// XXX: temporary interface
	void	ball_access_step( long *  self_access_step ,
				  long *  teammate_access_step ,
				  long *  opponent_access_step ,
				  long *  unknown_access_step ,
				  long  max_step = -1 ,
				  long  acc_level = -50 ) const;

	// XXX: temporary interface
	void	ball_access_step( long *  self_access_step ,
				  long *  teammate_access_step ,
				  Player_Set * teammate_set ,
				  long *  opponent_access_step ,
				  Player_Set * opponent_set ,
				  long *  unknown_access_step ,
				  Player_Set * unknown_set ,
				  long  max_step = -1 ,
				  long  acc_level = -50 ) const;


	//
	// XXX: below interfaces are temporal.
	//        This class should not have const set_*,
	//        These interfaces should not be here.
	//

	// XXX: temporal interface, use carefully!!
	void		set_target_teammate( int  num ) const /*XXX*/;
	// XXX: temporal interface
	bool		have_target_teammate() const;
	// XXX: temporal interface, use carefully!!
	int		get_target_teammate() const;

	// XXX: temporal interface, use carefully!!
	void		set_target_point( const D2_Vector &  point )
		const /*XXX*/;
	// XXX: temporal interface, use carefully!!
	bool		have_target_point() const;
	// XXX: temporal interface, use carefully!!
	D2_Vector	get_target_point() const;

	// XXX: temporal interface, use carefully!!
	void		set_message( const std::string &  message )
		const /*XXX*/;
	// XXX: temporal interface, use carefully!!
	bool		have_message() const;
	// XXX: temporal interface, use carefully!!
	std::string	get_message() const;

	// XXX: temporal interface, use carefully!!
	void		add_debug_line( const D2_Vector &  a ,
					const D2_Vector &  b ) const;
	// XXX: temporal interface, use carefully!!
	std::vector< std::pair<D2_Vector, D2_Vector> >
						get_debug_lines() const;

	// XXX: temporary interface
	bool	get_network_world_model_message
				( std::string * ,
				  const Soccer_Composite_Command & ) const;
};


#endif	/* FIELD_RECOG_INTERFACE_H_INCLUDED */
