#ifndef	   PLAYER_SET_H_INCLUDED
#define	   PLAYER_SET_H_INCLUDED

// Author:		H. Shimora
// Last-Modified:	Jan 18 2001
// Created:		Jan 18 2001
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Jan 18 2001    base version.
//
//

#include  "soccer_object_reference.h"
#include  "ref_count_ptr.h"
#include  <vector>

//-----------------------------------------------------------------------------
// abstract Player_Predicate definition
//-----------------------------------------------------------------------------

// Real Predicate Class
//   Inherit this to create new predicate.
class  Player_Predicate_Entity
{
public:
	virtual	~Player_Predicate_Entity();
	virtual	bool	operator() ( const Player_Reference & ) const;
};

// Wrapper class for Player_Predicate_Entity
class  Player_Predicate
{
private:
	ref_count_ptr<const Player_Predicate_Entity>	ent;

public:
	Player_Predicate
		( const ref_count_ptr<const Player_Predicate_Entity> & );

	Player_Predicate( const Player_Predicate_Entity * );

	/*non-virtual*/	bool	operator() ( const Player_Reference & ) const;
};


//-----------------------------------------------------------------------------
// abstract Player Set definition
//-----------------------------------------------------------------------------
class  Player_Set
{
#if 0
public:
	class  Player_Reference_Compare
	{
	public:
		bool	operator() ( const Player_Reference & ,
				     const Player_Reference & ) const;
	};
#endif

private:
	typedef std::vector<Player_Reference> Set_Type;

	Set_Type	pl_set;

public:
	typedef Set_Type::const_iterator	iterator;

public:
		Player_Set();

		Player_Set( const Field_Recog_Interface & ,
			    const Player_Predicate & );


	Player_Set::iterator	begin() const;
	Player_Set::iterator	end() const;

	size_t			size() const;
	bool			empty() const;

	void			clear();
	void			insert( const Player_Reference &  player );
};


//-----------------------------------------------------------------------------
// concrete Player_Predicate definition
//-----------------------------------------------------------------------------

//
// single Player_Predicate classes
//
class  All_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  All_Player_Except_Self_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Teammate_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Teammate_Except_Self_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Opponent_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Unknown_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Opponent_Or_Unknown_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};


class  Accurate_Teammate_Predicate : public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
	Accurate_Teammate_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Accurate_Teammate_Except_Self_Predicate : public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
	Accurate_Teammate_Except_Self_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Accurate_Opponent_Predicate : public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
	Accurate_Opponent_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Accurate_Opponent_Or_Unknown_Player_Predicate
					: public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
	Accurate_Opponent_Or_Unknown_Player_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}

	bool	operator() ( const Player_Reference & ) const;
};


class  Self_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Goalie_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Field_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Our_Team_Goalie_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Our_Team_Field_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Opponent_Team_Goalie_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Opponent_Team_Field_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  Coordinate_Accurate_Player_Predicate : public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
		Coordinate_Accurate_Player_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}
	bool	operator() ( const Player_Reference & ) const;
};

class  Velocity_Accurate_Player_Predicate : public Player_Predicate_Entity
{
private:
	long	accuracy_level;

public:
		Velocity_Accurate_Player_Predicate( long  accuracy_level )
			 : accuracy_level( accuracy_level ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Ball_Kickable_Player_Predicate : public Player_Predicate_Entity
{
protected:
	const Field_Recog_Interface &	f;

public:
		Ball_Kickable_Player_Predicate
			( const Field_Recog_Interface &  f )
			: f( f ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Point_Close_Player_Predicate : public Player_Predicate_Entity
{
protected:
	const Field_Recog_Interface &	f;
	const D2_Vector			point;
	const double			dist;

public:
		Point_Close_Player_Predicate
					( const Field_Recog_Interface &  f ,
					  const D2_Vector &  point ,
					  double  dist )
			: f( f ) , point( point ) , dist( dist ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Ball_Close_Player_Predicate : public Player_Predicate_Entity
{
private:
	const Field_Recog_Interface &	f;
	const double			dist;

public:
		Ball_Close_Player_Predicate
					( const Field_Recog_Interface &  f ,
					  double  dist )
			: f( f ) , dist( dist ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Self_Close_Player_Predicate : public Player_Predicate_Entity
{
private:
	const Field_Recog_Interface &	f;
	const double			dist;

public:
		Self_Close_Player_Predicate
					( const Field_Recog_Interface &  f ,
					  double  dist )
			: f( f ) , dist( dist ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Ball_Controllable_Player_Predicate : public Player_Predicate_Entity
{
private:
	const Field_Recog_Interface &	f;
	double				dist;

public:
		Ball_Controllable_Player_Predicate
					( const Field_Recog_Interface &  f ,
					  double  dist = 0.0 )
			: f( f ) , dist( dist ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Offside_Position_Player_Predicate : public Player_Predicate_Entity
{
public:
	bool	operator() ( const Player_Reference & ) const;
};

class  X_Coordinate_Forward_Player_Predicate : public Player_Predicate_Entity
{
private:
	double	threshold;

public:
	X_Coordinate_Forward_Player_Predicate( double  threshold )
			: threshold( threshold ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  X_Coordinate_Backward_Player_Predicate : public Player_Predicate_Entity
{
private:
	double	threshold;

public:
	X_Coordinate_Backward_Player_Predicate( double  threshold )
			: threshold( threshold ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Y_Coordinate_Plus_Player_Predicate : public Player_Predicate_Entity
{
private:
	double	threshold;

public:
	Y_Coordinate_Plus_Player_Predicate( double  threshold )
			: threshold( threshold ) {}

	bool	operator() ( const Player_Reference & ) const;
};

class  Y_Coordinate_Minus_Player_Predicate : public Player_Predicate_Entity
{
private:
	double	threshold;

public:
	Y_Coordinate_Minus_Player_Predicate( double  threshold )
			: threshold( threshold ) {}

	bool	operator() ( const Player_Reference & ) const;
};


//
// composite Player_Predicate classes
//
class  And_Player_Predicate : public Player_Predicate_Entity
{
private:
	std::vector<ref_count_ptr
			  <const Player_Predicate_Entity> >	predicates;

public:
	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p7 );

	And_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p7 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p8 );

	bool	operator() ( const Player_Reference & ) const;
};

class  Or_Player_Predicate : public Player_Predicate_Entity
{
private:
	std::vector<ref_count_ptr
			  <const Player_Predicate_Entity> >	predicates;

public:
	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p7 );

	Or_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p1 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p2 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p3 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p4 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p5 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p6 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p7 ,
		   const ref_count_ptr<const Player_Predicate_Entity> &  p8 );

	bool	operator() ( const Player_Reference & ) const;
};

class  Not_Player_Predicate : public Player_Predicate_Entity
{
private:
	const ref_count_ptr<const Player_Predicate_Entity>	p;

public:
	Not_Player_Predicate
		 ( const ref_count_ptr<const Player_Predicate_Entity> &  p )
		 : p( p ) {}

	bool	operator() ( const Player_Reference & ) const;
};


#endif	/* PLAYER_SET_H_INCLUDED */
