#ifndef	   SOCCER_OBJECT_REFERENCE_H_INCLUDED
#define	   SOCCER_OBJECT_REFERENCE_H_INCLUDED

// Author:		H. Shimora
// Last-Modified:	Jan 24 2001
// Created:		Jan 24 2001
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Jan 24 2001    base version.
//
//

#include  "field_recog_abstract.h"
#include  "field_momentary_info.h"
#include  "s_basic.h"
#include  <climits>
#include  <string>

class  Field_Recog_Interface;


//
// Soccer Object Reference
//
class  Soccer_Object_Reference
{
public:
	class  D2_Vector_with_Accuracy : public D2_Vector
	{
	public:
		enum{ INVALID_ACCURACY = LONG_MIN };

	private:
		long	acc;
		long	sight_acc;

	public:
		D2_Vector_with_Accuracy( const D2_Vector &  vec ,
					 long  acc ,  long  sight_acc )
			: D2_Vector( vec ) ,
			  acc( acc ) , sight_acc( sight_acc ) {}

		D2_Vector_with_Accuracy()
			: D2_Vector() ,
			  acc( INVALID_ACCURACY ) ,
			  sight_acc( INVALID_ACCURACY ){}

		long	accuracy_level() const
		{
			return( acc );
		}

		bool	accuracy_check( int  level = 0 ) const
		{
			return( acc >= level );
		}

		long	sight_accuracy_level() const
		{
			return( sight_acc );
		}

		bool	sight_accuracy_check( int  level = 0 ) const
		{
			return( sight_acc >= level );
		}

		bool	have_info() const
		{
			return( acc != INVALID_ACCURACY );
		}
	};

	class  D2_Region_with_Accuracy : public D2_Region
	{
	public:
		enum{ INVALID_ACCURACY = LONG_MIN };

	private:
		long	acc;
		long	sight_acc;

	public:
		D2_Region_with_Accuracy( const D2_Region &  reg ,
					 long  acc ,  long  sight_acc )
			: D2_Region( reg ) ,
			  acc( acc ) ,
			  sight_acc( sight_acc ) {}

		D2_Region_with_Accuracy()
			: D2_Region() ,
			  acc( INVALID_ACCURACY ) ,
			  sight_acc( INVALID_ACCURACY ) {}

		long	accuracy_level() const
		{
			return( acc );
		}

		bool	accuracy_check( int  level = 0 ) const
		{
			return( acc >= level );
		}

		long	sight_accuracy_level() const
		{
			return( sight_acc );
		}

		bool	sight_accuracy_check( int  level = 0 ) const
		{
			return( sight_acc >= level );
		}

		bool	have_info() const
		{
			return( acc != INVALID_ACCURACY );
		}
	};

	class  Angle_with_Validity : public Angle
	{
	public:
		enum{ INVALID_ACCURACY = LONG_MIN };

	private:
		bool	validity;

	public:
		Angle_with_Validity( const Angle &  ang ,  bool  validity )
			: Angle( ang ) , validity( validity ) {}

		Angle_with_Validity()
			: Angle( Angle::origin() ) ,
			  validity( INVALID_ACCURACY ) {}

		long	accuracy_level() const
		{
			return( validity ? 0 : INVALID_ACCURACY );
		}

		bool	accuracy_check( int = 0 ) const
		{
			return( validity );
		}

		bool	have_info() const
		{
			return( validity );
		}
	};

	class  Angle_Range_with_Validity : public Angle_Range
	{
	public:
		enum{ INVALID_ACCURACY = LONG_MIN };

	private:
		bool	validity;

	public:
		Angle_Range_with_Validity( const Angle_Range &  ang_range ,
					   bool  validity )
			: Angle_Range( ang_range ) , validity( validity ) {}

		Angle_Range_with_Validity()
			: Angle_Range( - Angle::half_circle() ,
				       + Angle::half_circle() ) ,
			  validity( INVALID_ACCURACY ) {}

		long	accuracy_level() const
		{
			return( validity ? 0 : INVALID_ACCURACY );
		}

		bool	accuracy_check( int = 0 ) const
		{
			return( validity );
		}

		bool	have_info() const
		{
			return( validity );
		}
	};

	class  Bool_with_Validity
	{
	public:
		enum{ INVALID_ACCURACY = LONG_MIN };

	private:
		bool	value;
		bool	validity;

	public:
		Bool_with_Validity( bool  value , bool  validity )
			: value( value ) , validity( validity ) {}

		operator bool() const
		{
			return( value );
		}

		Bool_with_Validity	operator !() const
		{
			return( Bool_with_Validity( ! value , validity ) );
		}

		long	accuracy_level() const
		{
			return( validity ? 0 : INVALID_ACCURACY );
		}

		bool	accuracy_check( int = 0 ) const
		{
			return( validity );
		}

		bool	have_info() const
		{
			return( validity );
		}
	};

protected:
	static	struct  Cache
	{
		D2_Vector_with_Accuracy		coordinate;
		D2_Region_with_Accuracy		coordinate_region;
		D2_Vector_with_Accuracy		velocity;
		D2_Region_with_Accuracy		velocity_region;
	} cache;

protected:
	const Field_Recog_Interface *	field;

protected:
	const Field_Recog_Abstract::Field_Recog_Snapshot&	snapshot()
									const;

public:
		 Soccer_Object_Reference( const Field_Recog_Interface &  f );
	virtual	~Soccer_Object_Reference();

	virtual	const D2_Vector_with_Accuracy &		coordinate() const;
	virtual	const D2_Region_with_Accuracy &		coordinate_region()
									 const;
	virtual	const D2_Vector_with_Accuracy &		velocity() const;
	virtual	const D2_Region_with_Accuracy &		velocity_region()
									 const;
};


//
// Ball
//
class  Ball_Reference : public Soccer_Object_Reference
{
protected:
	D2_Vector_with_Accuracy		cache_coordinate;
	D2_Region_with_Accuracy		cache_coordinate_region;
	D2_Vector_with_Accuracy		cache_velocity;
	D2_Region_with_Accuracy		cache_velocity_region;

public:
	void	update();

public:
	 Ball_Reference( const Field_Recog_Interface &  f );
	~Ball_Reference();

	const D2_Vector_with_Accuracy &		coordinate() const;
	const D2_Region_with_Accuracy &		coordinate_region() const;
	const D2_Vector_with_Accuracy &		velocity() const;
	const D2_Region_with_Accuracy &		velocity_region() const;

	double					ball_radius() const;
};


//
// Player
//
class  Team_Reference;

class  Player_Reference : public Soccer_Object_Reference
{
protected:
	bool				valid_flag;
	bool				unknown_flag;
	SObject_Player_Identifier	ident;
	int				unknown_index;
	SServer_Time			created_time;

	const Field_Recog_Abstract::Player_Info *	pl_cache;

protected:
	D2_Vector_with_Accuracy		cache_coordinate;
	D2_Region_with_Accuracy		cache_coordinate_region;
	D2_Vector_with_Accuracy		cache_velocity;
	D2_Region_with_Accuracy		cache_velocity_region;
	Angle_with_Validity		cache_body_angle;
	Angle_Range_with_Validity	cache_body_angle_range;
	Angle_with_Validity		cache_face_angle;
	Angle_Range_with_Validity	cache_face_angle_range;

protected:
	static	const Field_Recog_Abstract::Player_Info	unknown_player;
	const Field_Recog_Abstract::Player_Info &	player_info_no_cache()
									const;
	const Field_Recog_Abstract::Player_Info &	player_info() const;

public:
	void	update();

public:
	Player_Reference( const Field_Recog_Interface &  f );

	Player_Reference( const Field_Recog_Interface &  f ,
			  const S_Side &  s ,  int  num );

	Player_Reference( const Field_Recog_Interface &  f ,
			  const S_Side &  side ,  int  num ,
			  int  indx );

	~Player_Reference();

	Player_Reference &	operator= ( const Player_Reference & );

	bool			operator== ( const Player_Reference & ) const;

	bool			valid() const;

	S_Side			side() const;
	S_Side_LR		side_lr() const;
	Team_Reference		team() const;
	bool			teammate() const;
	bool			opponent() const;
	bool			unknown() const;
	bool			unknown_teammate() const;
	bool			unknown_opponent() const;
	int			player_number() const;
	bool			player_number_known() const;
	bool			self_player() const;
	Bool_with_Validity	goalie() const;
	Bool_with_Validity	field_player() const;
	Bool_with_Validity	actual_goalie() const;


	const D2_Vector_with_Accuracy &		coordinate() const;
	const D2_Region_with_Accuracy &		coordinate_region() const;
	const D2_Vector_with_Accuracy &		velocity() const;
	const D2_Region_with_Accuracy &		velocity_region() const;
	const Angle_with_Validity &		body_angle() const;
	const Angle_Range_with_Validity &	body_angle_range() const;
	const Angle_with_Validity &		face_angle() const;
	const Angle_Range_with_Validity &	face_angle_range() const;

	double			stamina() const;
	Double_Info		stamina_range() const;

	double			effort() const;
	Double_Info		effort_range() const;

	View_Width		view_width() const;

	double			ball_kickable_distance() const;
	double			ball_controllable_distance() const;

	bool			ball_kickable( double  offset = 0.0 ) const;
	bool			ball_kickable_may_be() const;
	bool			ball_controllable() const;

	bool			ball_tacklable_may_be() const;
	double			tackle_probability() const;

	double			player_size() const;

	bool			offside_position() const;
};


//
// Team
//
class  Team_Reference
{
protected:
	const Field_Recog_Interface *  field;
	S_Side	side_ident;

public:
		 Team_Reference( const Field_Recog_Interface &  f ,
				 const S_Side &  s );
	~Team_Reference();

	S_Side		side() const;
	S_Side_LR	side_lr() const;

	int		sign() const;	//    return -1 if our team
					// or return +1 opponent team
					// or return  0 unknown team

	int		score() const;

	std::string	team_name() const;

	Team_Reference	teammate_team() const;
	Team_Reference	opponent_team() const;

	double	offside_line() const;
	double	defence_line() const;
	double	forward_line() const;

	Player_Reference	goalie() const;

	D2_Vector	goal_coordinate() const;
	D2_Vector	positive_goal_post_coordinate() const;
	D2_Vector	negative_goal_post_coordinate() const;

	double		goal_line_x() const;
	double		goal_area_line_x() const;
	double		penalty_area_line_x() const;
};


//
// Play Mode
//
class  Play_Mode_Reference
{
protected:
	const Field_Recog_Interface *  field;

public:
		 Play_Mode_Reference( const Field_Recog_Interface &  f );
	~Play_Mode_Reference();

	Play_Mode			mode() const;
	long				start_time() const;
	Server_Raw_Whistle_Info		start_reason() const;

	bool	kickable_mode() const;
	bool	catchable_mode() const;
	bool	movable_mode() const;

	operator Play_Mode() const;
	bool	operator == ( const Play_Mode::Mode &  m ) const;
	bool	operator != ( const Play_Mode::Mode &  m ) const;
};


#if 0
//
// Player Type
//
class  Player_Type_Reference
{
protected:
	const Field_Recog_Interface *  field;

private:
	int	player_type_id;

public:
		 Player_Type_Reference( const Field_Recog_Interface &  f );
	~Player_Type_Reference();

	int	id() const;
	const Server_Raw_Player_Type_Info &	param() const;
};
#endif


#if 1
inline
const Soccer_Object_Reference::D2_Vector_with_Accuracy &
	Ball_Reference::coordinate() const
{
	return( this -> cache_coordinate );
}

inline
const Soccer_Object_Reference::D2_Region_with_Accuracy &
	Ball_Reference::coordinate_region() const
{
	return( this -> cache_coordinate_region );
}

inline
const Soccer_Object_Reference::D2_Vector_with_Accuracy &
	Ball_Reference::velocity() const
{
	return( this -> cache_velocity );
}

inline
const Soccer_Object_Reference::D2_Region_with_Accuracy &
	Ball_Reference::velocity_region() const
{
	return( this -> cache_velocity_region );
}

inline
const Soccer_Object_Reference::D2_Vector_with_Accuracy &
	Player_Reference::coordinate() const
{
	return( this -> cache_coordinate );
}

inline
const Soccer_Object_Reference::D2_Region_with_Accuracy &
	Player_Reference::coordinate_region() const
{
	return( this -> cache_coordinate_region );
}

inline
const Soccer_Object_Reference::D2_Vector_with_Accuracy &
	Player_Reference::velocity() const
{
	return( this -> cache_velocity );
}

inline
const Soccer_Object_Reference::D2_Region_with_Accuracy &
	Player_Reference::velocity_region() const
{
	return( this -> cache_velocity_region );
}

inline
const Soccer_Object_Reference::Angle_with_Validity &
	Player_Reference::body_angle() const
{
	return( this -> cache_body_angle );
}

inline
const Soccer_Object_Reference::Angle_Range_with_Validity &
	Player_Reference::body_angle_range() const
{
	return( this -> cache_body_angle_range );
}

inline
const Soccer_Object_Reference::Angle_with_Validity &
	Player_Reference::face_angle() const
{
	return( this -> cache_face_angle );
}

inline
const Soccer_Object_Reference::Angle_Range_with_Validity &
	Player_Reference::face_angle_range() const
{
	return( this -> cache_face_angle_range );
}
#endif


#endif	/* SOCCER_OBJECT_REFERENCE_H_INCLUDED */
