#ifndef	   COMMUNICATION_CANNEL_H_INCLUDED
#define	   COMMUNICATION_CANNEL_H_INCLUDED

// Author:		H. Shimora
// Created:		Feb  4 2001
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  Feb  4 2001    base version.
//
//

#include  "ip_address.h"
#include  "ref_count_ptr.h"
#include  <string>
#ifdef HAVE_ZLIB
# include  <zlib.h>
#endif

template<class Connection_T>
class  Communication_Cannel
{
public:
	static	const	long	DEFAULT_MAX_RESPONCE_TIME
						= 5 * 1000 * 1000; // 5 sec.

protected:
	Connection_T	con;
	int		compression_level;

#ifdef HAVE_ZLIB
	z_stream	compress_z;
	z_stream	decompress_z;
#endif

protected:
	void	compress  ( const std::string &  from ,  std::string &  to );
	void	decompress( const std::string &  from ,  std::string &  to );

private:
	// Don't allow copy constructor
	 Communication_Cannel( const Communication_Cannel & );
	// Don't allow copy
	 Communication_Cannel &	 operator=( const Communication_Cannel & );

public:
		 Communication_Cannel( const std::string &  host ,
				       port_number_t  port ,
				       bool  debug ,
				       size_t  max_msg_len ,
				       long  max_response
					       = DEFAULT_MAX_RESPONCE_TIME ,
				       bool  close_on_exec = false );

		 Communication_Cannel( port_number_t  port ,
				       bool  debug ,
				       size_t  max_msg_len ,
				       long  max_response
					       = DEFAULT_MAX_RESPONCE_TIME ,
				       bool  close_on_exec = false );

	virtual	~Communication_Cannel();

		void		set_compression_level( int  level );

				operator bool() const;
		bool		responsive() const;

		int		send( const std::string &  mes );

		bool		recv( ref_count_ptr<std::string> *  data ,
				      bool  block = false );

		bool		recv( ref_count_ptr<std::string> *  data ,
				      long  usec );

		bool		recv( std::string *  data ,
				      bool  block = false );

		bool		recv( std::string *  data ,
				      long  usec );
};


#include  "communication_cannel_inline.cc"

#endif	/* COMMUNICATION_CANNEL_H_INCLUDED */
