#include  "communication_cannel.h"
#include  "debugstream.h"
#include  <iostream>

template<class Connection_T>
Communication_Cannel<Connection_T>::Communication_Cannel
			   ( const std::string &  host ,
			     port_number_t  port ,
			     bool  debug ,
			     size_t  max_msg_len ,
			     long  max_response ,
			     bool  close_on_exec )
	: con( host , port , debug , max_msg_len , max_response ,
	       close_on_exec ) ,
	  compression_level( 0 )
#ifdef HAVE_ZLIB
	  , compress_z() , decompress_z()
#endif
{
}

template<class Connection_T>
Communication_Cannel<Connection_T>::Communication_Cannel
			   ( port_number_t  port ,
			     bool  debug ,
			     size_t  max_msg_len ,
			     long  max_response ,
			     bool  close_on_exec )
	: con( port , debug , max_msg_len , max_response ,
	       close_on_exec ) ,
	  compression_level( 0 )
#ifdef HAVE_ZLIB
	  , compress_z() , decompress_z()
#endif
{
}

template<class Connection_T>
Communication_Cannel<Connection_T>::~Communication_Cannel()
{
#ifdef HAVE_ZLIB
	if ( this -> compression_level != 0 )
	{
		for(;;)
		{
			static_cast<void *>( this -> compress_z.next_in )
				= 0;
			this -> compress_z.avail_in = 0;

			char	ch;
			static_cast<void *>( this -> compress_z.next_out )
				= &ch;
			this -> compress_z.avail_out = 1;

			int	status = deflate( &(this -> compress_z) ,
						  Z_FINISH );

			if ( status == Z_OK )
			{
				continue;
			}
			else if ( status == Z_STREAM_END )
			{
				break;
			}
			else
			{
				std::cerr << "deflate failed: " << status
					  << "["
					  << (this -> compress_z.msg ?
					      this -> compress_z.msg : "")
					  << "]" << std::endl;
				break;
			}
		}

		if ( deflateEnd( &(this -> compress_z) ) != Z_OK )
		{
			std::cerr << "deflateEnd failed: "
				  << "[" << (this -> compress_z.msg ?
					     this -> compress_z.msg : "")
				  << "]" << std::endl;
		}


		if ( inflateEnd( &this -> decompress_z ) != Z_OK )
		{
			std::cerr << "inflateEnd failed: "
				  << "[" << (this -> decompress_z.msg ?
					     this -> decompress_z.msg : "")
				  << "]"
				  << std::endl;
		}
	}
#endif
}

template<class Connection_T>
void   Communication_Cannel<Connection_T>::set_compression_level( int  level )
{
	this -> compression_level = level;

#ifdef HAVE_ZLIB
	this -> compress_z.zalloc = Z_NULL;
	this -> compress_z.zfree  = Z_NULL;
	this -> compress_z.opaque = Z_NULL;

	this -> decompress_z.zalloc = Z_NULL;
	this -> decompress_z.zfree  = Z_NULL;
	this -> decompress_z.opaque = Z_NULL;


	if ( deflateInit( &(this -> compress_z) ,
			  this -> compression_level ) != Z_OK )
	{
		std::cerr << "deflateInit failed: "
			  << "[" << (this -> compress_z.msg ?
				     this -> compress_z.msg : "") << "]"
			  << std::endl;

		this -> compression_level = 0;

		return;
	}


	if ( inflateInit( &(this -> decompress_z) ) != Z_OK )
	{
		std::cerr << "inflateInit failed: "
			  << "[" << (this -> decompress_z.msg ?
				     this -> decompress_z.msg : "") << "]"
			  << std::endl;

		this -> compression_level = 0;

		return;
	}
#endif
}

template<class Connection_T>
Communication_Cannel<Connection_T>::operator bool() const
{
	return( this -> con.operator bool() );
}

template<class Connection_T>
bool   Communication_Cannel<Connection_T>::responsive() const
{
	return( this -> con.responsive() );
}

template<class Connection_T>
int    Communication_Cannel<Connection_T>::send( const std::string &  mes )
{
	if ( this -> compression_level == 0 )
	{
		return( this -> con.send( mes ) );
	}
	else
	{
		std::string	compressed_message;

		this -> compress( mes , compressed_message );

		return( this -> con.send( compressed_message ) );
	}
}


template<class Connection_T>
bool   Communication_Cannel<Connection_T>
				::recv( ref_count_ptr<std::string> *  data ,
					bool  block )
{
	if ( this -> compression_level == 0 )
	{
		(void)(this -> con.recv( data  , block ));
	}
	else
	{
		std::string	compressed_message;

		(void)(this -> con.recv( &compressed_message , block ));

		*data = new std::string;

		this -> decompress( compressed_message , **data );
	}

	return( (*data) -> length() != 0 );
}

template<class Connection_T>
bool   Communication_Cannel<Connection_T>
				::recv( ref_count_ptr<std::string> *  data ,
					long  usec )
{
	if ( this -> compression_level == 0 )
	{
		(void)(this -> con.recv( data , usec ));
	}
	else
	{
		std::string	compressed_message;

		(void)(this -> con.recv( &compressed_message , usec ));

		*data = new std::string;

		this -> decompress( compressed_message , **data );
	}

	return( (*data) -> length() != 0 );
}

template<class Connection_T>
bool   Communication_Cannel<Connection_T>::recv( std::string *  data ,
						 bool  block )
{
	if ( this -> compression_level == 0 )
	{
		(void)(this -> con.recv( data , block ));
	}
	else
	{
		std::string	compressed_message;

		(void)(this -> con.recv( &compressed_message , block ));

		this -> decompress( compressed_message , *data );
	}

	return( data -> length() != 0 );
}

template<class Connection_T>
bool   Communication_Cannel<Connection_T>::recv( std::string *  data ,
						 long  usec )
{
	if ( this -> compression_level == 0 )
	{
		(void)(this -> con.recv( data , usec ));
	}
	else
	{
		std::string	compressed_message;

		(void)(this -> con.recv( &compressed_message , usec ));

		this -> decompress( compressed_message , **data );
	}

	return( data -> length() != 0 );
}


template<class Connection_T>
void   Communication_Cannel<Connection_T>::compress
					   ( const std::string &  from ,
					     std::string &  to )
{
#ifdef HAVE_ZLIB
	if ( Debug_Stream::dbg )
	{
		Debug_Stream::dbg << "message = [" << from << "]" << std::endl;
	}

	to.resize( 0 );

	static_cast<void *>( this -> compress_z.next_in ) = from.data();
	this -> compress_z.avail_in = from.size();

	char	out_buf[BUFSIZ];
	static_cast<void *>( this -> compress_z.next_out ) = out_buf;
	this -> compress_z.avail_out = sizeof(out_buf);

	for(;;)
	{
		int	status = deflate( &(this -> compress_z) ,
					  Z_SYNC_FLUSH );

		if ( status == Z_OK
		  || status == Z_STREAM_END )
		{
			to.append( out_buf ,
				   sizeof(out_buf)
				   - this -> compress_z.avail_out );

			static_cast<void *>( this -> compress_z.next_out )
								    = out_buf;
			this -> compress_z.avail_out = sizeof(out_buf);

			if ( status == Z_STREAM_END )
			{
				break;
			}
		}
		else if ( status == Z_BUF_ERROR )
		{
			break;
		}
		else
		{
			std::cerr << "deflate failed: " << status
				  << "[" << (this -> compress_z.msg ?
					     this -> compress_z.msg : "")
				  << "]" << std::endl;
			break;
		}
	}
#else
	to = from;
#endif
}

template<class Connection_T>
void   Communication_Cannel<Connection_T>::decompress
					   ( const std::string &  from ,
					     std::string &  to )
{
#ifdef HAVE_ZLIB
	to.resize( 0 );

	static_cast<void *>( this -> decompress_z.next_in ) = from.data();
	this -> decompress_z.avail_in = from.size();

	char	out_buf[BUFSIZ];
	static_cast<void *>( this -> decompress_z.next_out ) = out_buf;
	this -> decompress_z.avail_out = sizeof(out_buf);

	for(;;)
	{
		int	status = inflate( &(this -> decompress_z) ,
					  Z_SYNC_FLUSH );

		if ( status == Z_OK
		  || status == Z_STREAM_END )
		{
			to.append( out_buf ,
				   sizeof(out_buf)
				   - this -> decompress_z.avail_out );

			static_cast<void *>( this -> decompress_z.next_out )
								    = out_buf;
			this -> decompress_z.avail_out = sizeof(out_buf);

			if ( Debug_Stream::dbg )
			{
				Debug_Stream::dbg << "decompressed message = ["
						  << to << "]" << std::endl;
			}

			if ( status == Z_STREAM_END )
			{
				break;
			}
		}
		else if ( status == Z_BUF_ERROR )
		{
			break;
		}
		else
		{
			std::cerr << "inflate failed: "
				  << "[" << (this -> decompress_z.msg ?
					     this -> decompress_z.msg : "")
				  << "]" << std::endl;
			break;
		}
	}
#else
	to = from;
#endif
}
