#ifndef	   MONITOR_VIEW_DATA_H_INCLUDED
#define	   MONITOR_VIEW_DATA_H_INCLUDED

// Author:		H. Shimora
// Created:		May 22 2000
// Version:		0.00

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.00  May 22 2000    base version.
//
//

#include  "s_basic.h"
#include  "angle.h"
#include  "d2_vector.h"
#include  "sserver_monitor_log_format.h"
#include  <vector>
#include  <string>
#include  <iostream>

class  Monitor_View_Data
{
public:
	class  Player_State
	{
	protected:
		SServer_Monitor_Log_Format::player_t	p;

		S_Side_LR	side_;
		char		uniform_number_;
		bool		have_showinfo_t2_extension_ : 1;
		bool		have_overrided_stamina_     : 1;
		bool		have_overrided_effort_      : 1;
		bool		have_overrided_head_angle_  : 1;
		bool		have_overrided_view_width_  : 1;

	public:
		Player_State();
		Player_State( const SServer_Monitor_Log_Format::player_t & );
		Player_State( const SServer_Monitor_Log_Format::pos_t & );

		bool	have_stamina() const;
		bool	have_effort() const;
		bool	have_head_angle() const;
		bool	have_view_width() const;

		bool	valid()		  const;

		bool	goalie()	  const;
		bool	kicking()	  const;
		bool	kicking_fault()	  const;
		bool	catching()	  const;
		bool	catching_fault()  const;
		bool	ball_to_player()  const;
		bool	player_to_ball()  const;
		bool	discard()	  const;
		bool	lost()		  const;
		bool	ball_collide()	  const;
		bool	player_collide()  const;
		bool	tackling()	  const;
		bool	tackling_fault()  const;
		bool	back_pass()	  const;
		bool	free_kick_fault() const;


		S_Side_LR	side()	 const;

		int	uniform_number() const;

		double	x()		 const;
		double	y()		 const;

		Angle	body_angle()	 const;
		Angle	head_angle()	 const;

		Angle	view_width()	 const;

		double	stamina()	 const;
		double	effort()	 const;
		bool	full_effort()	 const;

	public:
		void	set_valid          ( bool  b );
		void	set_goalie         ( bool  b );
		void	set_kicking        ( bool  b );
		void	set_kicking_fault  ( bool  b );
		void	set_catching       ( bool  b );
		void	set_catching_fault ( bool  b );
		void	set_ball_to_player ( bool  b );
		void	set_player_to_ball ( bool  b );
		void	set_discard        ( bool  b );
		void	set_lost           ( bool  b );
		void	set_ball_collide   ( bool  b );
		void	set_player_collide ( bool  b );
		void	set_tackling       ( bool  b );
		void	set_tackling_fault ( bool  b );
		void	set_back_pass      ( bool  b );
		void	set_free_kick_fault( bool  b );


		void	set_uniform_number( int  u );

		void	set_side( const S_Side_LR &  s );

		void	set_body_angle( const Angle &  a );
		void	set_x( double  d );
		void	set_y( double  d );

		void	set_head_angle( const Angle & );
		void	set_view_width( const Angle & );
		void	set_stamina( double  s );
		void	set_effort( double  e );
	};

	struct  Team_State
	{
		std::string	team_name;
		long		score;

	public:
		 Team_State() : team_name("") , score(0) {}
		~Team_State() {}
	};

	struct  Monitor_Play_Mode
	{
		char	pmode;

	public:
		enum Mode
		{
		    Null_Mode ,
		    Before_Kick_Off ,
		    Time_Over ,
		    Play_On ,
		    Kick_Off_Left ,             Kick_Off_Right ,
		    Kick_In_Left ,              Kick_In_Right ,
		    Free_Kick_Left ,            Free_Kick_Right ,
		    Corner_Kick_Left ,          Corner_Kick_Right ,
		    Goal_Kick_Left ,            Goal_Kick_Right ,
		    Goal_Left ,                 Goal_Right ,
		    Drop_Ball ,
		    Offside_Left ,              Offside_Right ,
		    Penalty_Kick_Left ,         Penalty_Kick_Right ,
		    First_Half_Over ,
		    Pause ,
		    Human_Judge ,
		    Foul_Charge_Left ,          Foul_Charge_Right ,
		    Foul_Push_Left ,            Foul_Push_Right ,
		    Foul_Multiple_Attack_Left , Foul_Multiple_Attack_Right ,
		    Foul_Ballout_Left ,         Foul_Ballout_Right ,
		    Back_Pass_Left ,            Back_Pass_Right ,
		    Free_Kick_Fault_Left ,      Free_Kick_Fault_Right ,
		    Catch_Fault_Left,           Catch_Fault_Right ,
		    Indirect_Free_Kick_Left ,   Indirect_Free_Kick_Right ,
		    Penalty_Setup_Left ,        Penalty_Setup_Right ,
		    Penalty_Ready_Left ,        Penalty_Ready_Right ,
		    Penalty_Taken_Left ,        Penalty_Taken_Right ,
		    Penalty_Miss_Left ,         Penalty_Miss_Right ,
		    Penalty_Score_Left ,        Penalty_Score_Right
		};

		static const int	N_MODE = 50;

	public:
		Monitor_Play_Mode( char  m = Null_Mode ) : pmode(m) {}

		operator Monitor_Play_Mode::Mode() const
		{
			return( static_cast<Mode>(pmode) );
		}

		bool	operator == ( const Mode &  m ) const
		{
			return( pmode == static_cast<char>(m) );
		}

		bool	operator != ( const Mode &  m ) const
		{
			return( ! ((*this) == m) );
		}

		bool	operator == ( const Monitor_Play_Mode &  m ) const
		{
			return( pmode == m.pmode );
		}

		bool	operator != ( const Monitor_Play_Mode &  m ) const
		{
			return( ! ((*this) == m) );
		}

		std::string	play_mode_str
				( char  sep = ' ' ,
				  const std::string &  left_team_name = "" ,
				  const std::string &  right_team_name = "" )
									 const
		{
			struct  Mode_Trans_Table
			{
				const char *	mode_string;
				S_Side_LR	side;
			};

			static	Mode_Trans_Table	table[N_MODE]
			= {{ "null" ,                 S_Side_LR::Unknown } ,
			   { "before kick off" ,      S_Side_LR::Unknown } ,
			   { "time over" ,            S_Side_LR::Unknown } ,
			   { "play on" ,              S_Side_LR::Unknown } ,
			   { "kick off" ,             S_Side_LR::Left_Side } ,
			   { "kick off" ,             S_Side_LR::Right_Side } ,
			   { "kick in" ,              S_Side_LR::Left_Side } ,
			   { "kick in" ,              S_Side_LR::Right_Side } ,
			   { "free kick" ,            S_Side_LR::Left_Side } ,
			   { "free kick" ,            S_Side_LR::Right_Side } ,
			   { "corner kick" ,          S_Side_LR::Left_Side } ,
			   { "corner kick" ,          S_Side_LR::Right_Side } ,
			   { "goal kick" ,            S_Side_LR::Left_Side } ,
			   { "goal kick" ,            S_Side_LR::Right_Side } ,
			   { "goal" ,                 S_Side_LR::Left_Side } ,
			   { "goal" ,                 S_Side_LR::Right_Side } ,
			   { "drop ball" ,            S_Side_LR::Unknown } ,
			   { "offside" ,              S_Side_LR::Left_Side } ,
			   { "offside" ,              S_Side_LR::Right_Side } ,
			   { "penalty kick" ,         S_Side_LR::Left_Side } ,
			   { "penalty kick" ,         S_Side_LR::Right_Side } ,
			   { "first half over" ,      S_Side_LR::Unknown } ,
			   { "pause" ,                S_Side_LR::Unknown } ,
			   { "human judge" ,          S_Side_LR::Unknown } ,
			   { "foul charge" ,          S_Side_LR::Left_Side } ,
			   { "foul charge" ,          S_Side_LR::Right_Side } ,
			   { "foul push" ,            S_Side_LR::Left_Side } ,
			   { "foul push" ,            S_Side_LR::Right_Side } ,
			   { "foul multiple_attack" , S_Side_LR::Left_Side } ,
			   { "foul multiple_attack" , S_Side_LR::Right_Side } ,
			   { "foul ballout" ,         S_Side_LR::Left_Side } ,
			   { "foul ballout" ,         S_Side_LR::Right_Side } ,
			   { "back pass" ,            S_Side_LR::Left_Side } ,
			   { "back pass" ,            S_Side_LR::Right_Side } ,
			   { "free kick fault" ,      S_Side_LR::Left_Side } ,
			   { "free kick fault" ,      S_Side_LR::Right_Side } ,
			   { "catch fault" ,          S_Side_LR::Left_Side } ,
			   { "catch fault" ,          S_Side_LR::Right_Side } ,
			   { "indirect free kick" ,   S_Side_LR::Left_Side } ,
			   { "indirect free kick" ,   S_Side_LR::Right_Side } ,
			   { "penalty setup" ,        S_Side_LR::Left_Side } ,
			   { "penalty setup" ,        S_Side_LR::Right_Side } ,
			   { "penalty ready" ,        S_Side_LR::Left_Side } ,
			   { "penalty ready" ,        S_Side_LR::Right_Side } ,
			   { "penalty taken" ,        S_Side_LR::Left_Side } ,
			   { "penalty taken" ,        S_Side_LR::Right_Side } ,
			   { "penalty miss" ,         S_Side_LR::Left_Side } ,
			   { "penalty miss" ,         S_Side_LR::Right_Side } ,
			   { "penalty score" ,        S_Side_LR::Left_Side } ,
			   { "penalty score" ,        S_Side_LR::Right_Side }
			};

			if ( static_cast<size_t>(pmode)
			       >= (sizeof(table) / sizeof(table[0]))
			  || pmode < 0 )
			{
				std::cerr << "Unknown play mode "
					  << static_cast<int>( pmode )
					  << std::endl;

				return( "Unknown" );
			}


			std::string	left_str;
			if ( left_team_name.length() == 0 )
			{
				left_str = "left";
			}
			else
			{
				left_str = std::string("\"")
					   + left_team_name + "\"";
			}

			std::string	right_str;
			if ( right_team_name.length() == 0 )
			{
				right_str = "right";
			}
			else
			{
				right_str = std::string("\"")
					    + right_team_name + "\"";
			}

			const Mode_Trans_Table &
				m = table[static_cast<int>(pmode)];

			std::string	m_str;

			for ( size_t  i = 0  ;
			      m.mode_string[i] != '\0'  ;
			      i ++ )
			{
				if ( m.mode_string[i] == ' ' )
				{
					m_str += sep;
				}
				else
				{
					m_str += m.mode_string[i];
				}
			}


			if ( m.side == S_Side_LR::Left_Side )
			{
				return( m_str + sep + left_str );
			}
			else if ( m.side == S_Side_LR::Right_Side )
			{
				return( m_str + sep + right_str );
			}
			else
			{
				return( m_str );
			}
		}

		S_Side_LR	advantage() const
		{
			switch( pmode )
			{
			case Kick_Off_Left:
			case Kick_In_Left:
			case Free_Kick_Left:
			case Corner_Kick_Left:
			case Goal_Kick_Left:
			case Offside_Right:
			case Penalty_Kick_Right:
			case Foul_Charge_Right:
			case Foul_Push_Right:
			case Foul_Multiple_Attack_Right:
			case Foul_Ballout_Right:
			case Back_Pass_Right:
			case Free_Kick_Fault_Right:
			case Indirect_Free_Kick_Left:
			case Penalty_Setup_Left:
			case Penalty_Ready_Left:
				return( S_Side_LR::Left_Side );
				break;

			case Kick_Off_Right:
			case Kick_In_Right:
			case Free_Kick_Right:
			case Corner_Kick_Right:
			case Goal_Kick_Right:
			case Offside_Left:
			case Penalty_Kick_Left:
			case Foul_Charge_Left:
			case Foul_Push_Left:
			case Foul_Multiple_Attack_Left:
			case Foul_Ballout_Left:
			case Back_Pass_Left:
			case Free_Kick_Fault_Left:
			case Indirect_Free_Kick_Right:
			case Penalty_Setup_Right:
			case Penalty_Ready_Right:
				return( S_Side_LR::Right_Side );
				break;

			case Null_Mode:
			case Before_Kick_Off:
			case Time_Over:
			case Play_On:
			case Goal_Left:
			case Goal_Right:
			case Drop_Ball:
			default:
				return( S_Side_LR::Unknown );
				break;
			}
		}

		static	std::vector<Monitor_Play_Mode>	mode_list()
		{
			std::vector<Monitor_Play_Mode>	m_list;

			for( int  i = 0  ;  i < N_MODE  ;  i ++ )
			{
				m_list.push_back( Mode(i) );
			}

			return( m_list );
		}
	};


public:
	Monitor_Play_Mode	play_mode;
	long			time_count;
	D2_Vector		ball;
	Player_State		player[MAX_PLAYER * 2];
	Team_State		team[2];

public:
	 Monitor_View_Data()
		 : play_mode() ,
		   time_count( -1 ) ,
		   ball( 0.0 , 0.0 ) ,
		   player(),
		   team()
	{
	}

	~Monitor_View_Data()
	{
	}
};


#endif	/* MONITOR_VIEW_DATA_H_INCLUDED */
