#ifdef SSERVER_PLAYER_CONNECTION_TEST
#include  "sserver_player_connection.h"

int    main( void )
{
	SServer_Player_Connection  server;

	server.send_init( "test" );

	server.send_move( -3.0 , 0.12345678 );

	for(;;)
	{
		Information_from_Server	info;

		server.recv_info( &info );
	}

	return( 0 );
}
#endif




#include  "sserver_player_connection.h"
#include  "sserver_message_translator.h"
#include  "adjust_sleep.h"
#include  "debugstream.h"
#include  <iostream>

using namespace std;

const	double  SServer_Player_Connection::DEFAULT_WAIT_TIME_STEP = 1.1;

SServer_Player_Connection::SServer_Player_Connection
					( const SServer_Param &  param ,
					  const string &  host ,
					  port_number_t  port ,
					  int  compression_level ,
					  long  max_response ,
					  long  default_wait_time ,
					  Parse_Policy  p_policy )
	: Communication_Cannel<UDP_Connection>
		( host ,
		  port ,
		  true , // with debug output
		  SServer_Param::SSERVER_MAX_MESSAGE_LENGTH ,
		  max_response ) ,
	  param( param ) ,
	  our_team_name( "" ) ,
	  opponent_team_name( "" ) ,
	  our_side( S_Side_LR::Unknown ) ,
	  self_player_number( -1 ) ,
	  default_wait_time( default_wait_time ) ,
	  parse_policy( p_policy ) ,
	  compression_level( compression_level ) ,
	  waiting_for_compression_ok( false ) ,
	  send_time( Time_Stamp::Current_Time )
{
}


SServer_Player_Connection::~SServer_Player_Connection()
{
}

int    SServer_Player_Connection::fd() const
{
	return( con.fd() );
}

const std::string &  SServer_Player_Connection::get_opponent_team_name() const
{
	return( this -> opponent_team_name );
}

long   SServer_Player_Connection::wait_time() const
{
	if ( this -> default_wait_time == -1 )
	{
		return( static_cast<long>( param.SIMULATOR_STEP() * 1000
					   * DEFAULT_WAIT_TIME_STEP ) );
	}
	else
	{
		return( this -> default_wait_time );
	}
}

long   SServer_Player_Connection::default_wait() const
{
	return( this -> wait_time() );
}

bool   SServer_Player_Connection::send_ok() const
{
	return( this -> compression_level == 0
		|| ! (this -> waiting_for_compression_ok) );
}


int    SServer_Player_Connection::send_with_interval( const string &  str ,
						      long  interval_usec )
{
	if ( interval_usec != 0 )
	{
		Time_Stamp	current_time( Time_Stamp::Current_Time );

		long	diff = (current_time - send_time).usec();

		if ( diff  <  interval_usec )
		{
			Adjust_Sleep::usleep( interval_usec - diff );
		}
	}

	int	n = this -> send( str );

	this -> send_time.set_current_time();

	// XXX
	// compare int and size_t
	//
	if ( static_cast<size_t>(n) == str.length() )
	{
		return( n );
	}
	else
	{
		return( -1 );
	}
}


void   SServer_Player_Connection::send_compression()
{
	if ( this -> compression_level != 0 )
	{
		const string	send_string
				= SServer_Message_Translator
					::compression_level_command
						( this -> compression_level );

		this -> waiting_for_compression_ok = true;

		this -> send_with_interval( send_string , 200 * 1000 );
	}
}


int    SServer_Player_Connection::send_init( const string &  our_team_name ,
					     bool  goalie ,
					     S_Side_LR *  our_side ,
					     int *  self_player_number ,
					     Play_Mode *  play_mode ,
					     int  major_version ,
					     int  minor_version )
{
	const string	send_string = SServer_Message_Translator::init_command
							( our_team_name ,
							  goalie ,
							  major_version ,
							  minor_version );


	int	ret = this -> send( send_string );

	ref_count_ptr<string>	buf;
	for(;;)
	{
		(void)(this -> con.recv( &buf ,
					 /* block */ true ,
					 /* reply */ true ));

		if ( buf -> length() != 0 )
		{
			break;
		}

		if ( ! (this -> responsive()) )
		{
			return( -1 );
		}
	}


	SServer_Message_Translator::parse_init_message( our_side ,
							self_player_number ,
							play_mode ,
							buf );

	this -> our_team_name      = our_team_name;
	this -> opponent_team_name = "";
	this -> our_side           = *our_side;
	this -> self_player_number = *self_player_number;

	this -> send_compression();

	return( ret );
}


int    SServer_Player_Connection::send_reconnect
					( const string &  our_team_name ,
					  int  player_number ,
					  S_Side_LR *  our_side ,
					  Play_Mode *  play_mode ,
					  long  interval_usec )
{
	const string	send_string
			 = SServer_Message_Translator::reconnect_command
							    ( our_team_name ,
							      player_number );

	int	ret = this -> send_with_interval( send_string ,
						  interval_usec );

	ref_count_ptr<string>	buf;
	for(;;)
	{
		(void)(this -> con.recv( &buf , true , true ));

		if ( buf -> length() != 0 )
		{
			break;
		}

		if ( ! (this -> responsive()) )
		{
			return( -1 );
		}
	}


	SServer_Message_Translator::parse_reconnect_message
				      ( our_side , play_mode , buf );

	this -> our_team_name      = our_team_name;
	this -> opponent_team_name = "";
	this -> our_side           = *our_side;
	this -> self_player_number = player_number;

	this -> send_compression();

	return( ret );
}

int    SServer_Player_Connection::send_reconnect( const string &  team_name ,
						  int  player_number ,
						  S_Side_LR *  our_side ,
						  Play_Mode *  play_mode ,
						  bool  default_wait )
{
	return( send_reconnect( team_name , player_number ,
				our_side , play_mode ,
				(default_wait ? (this -> wait_time()) : 0) ) );
}


int    SServer_Player_Connection::send_bye( long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::bye_command();

	return( this -> send_with_interval( send_string , interval_usec ) );
}

int    SServer_Player_Connection::send_bye( bool  default_wait )
{
	return( send_bye( (default_wait ? (this -> wait_time()) : 0) ) );
}



int    SServer_Player_Connection::send_command
				  ( const Soccer_Command &  com ,
				    long  interval_usec )
{
	switch( com.type() )
	{
	case Soccer_Command::Move_Command:
	case Soccer_Command::Dash_Command:
	case Soccer_Command::Kick_Command:
	case Soccer_Command::Turn_Command:
	case Soccer_Command::Tackle_Command:
	case Soccer_Command::Catch_Ball_Command:
	case Soccer_Command::Change_Player_Type_Command:
	case Soccer_Command::Turn_Neck_Command:
	case Soccer_Command::Change_View_Command:
	case Soccer_Command::Sense_Body_Command:
	case Soccer_Command::Say_Command:
	case Soccer_Command::Point_To_Command:
	case Soccer_Command::Bye_Command:
		return( this -> send_with_interval
			( SServer_Message_Translator::command( com ) ,
			  interval_usec ) );
		break;

	case Soccer_Command::Attention_To_Command:
		if ( com.attention_to_team() == S_Side::Unknown )
		{
			Debug_Stream::dbg << "Can't send Attention_To: "
					  << "unknown team player"
					  << std::endl;
			break;
		}

		return( this -> send_with_interval
			( SServer_Message_Translator::command( com ) ,
			  interval_usec ) );
		break;

	case Soccer_Command::Wait_Command:
	case Soccer_Command::No_Command_Command:
	case Soccer_Command::Illegal_Command:
		break;
	}

	return( -1 );
}

int    SServer_Player_Connection::send_command
				    ( const Soccer_Command &  command ,
				      bool  default_wait )
{
	return( send_command( command ,
			      (default_wait ? (this -> wait_time()) : 0) ) );
}




bool   SServer_Player_Connection::recv_info( Server_Raw_Info *  info ,
					     bool  block )
{
	ref_count_ptr<string>	data;
	this -> recv( &data , block );

	SServer_Message_Translator::parse_message
		( info ,
		  ((opponent_team_name.length() == 0) ?
		   &opponent_team_name : static_cast<std::string *>(0)) ,
		  data ,
		  our_side ,
		  our_team_name ,
		  self_player_number ,
		  (parse_policy == Fast_Parse) );

	this -> process_info( *info );

	return( ! (info -> no_info()) );
}


bool   SServer_Player_Connection::recv_info( Server_Raw_Info *  info ,
					     long  usec )
{
	ref_count_ptr<string>	message;
	this -> recv( &message , usec );

	SServer_Message_Translator::parse_message
		( info ,
		  ((opponent_team_name.length() == 0) ?
		   &opponent_team_name : static_cast<std::string *>(0)) ,
		  message ,
		  our_side ,
		  our_team_name ,
		  self_player_number ,
		  (parse_policy == Fast_Parse) );

	this -> process_info( *info );

	return( ! (info -> no_info()) );
}


void   SServer_Player_Connection::process_info( const Server_Raw_Info &  info )
{
	if ( info.type() == Server_Raw_Info::Type_Unknown )
	{
		cerr << "Unknown format: [" << info.raw_message() << "]"
		     << endl;
	}
	else if ( info.type() == Server_Raw_Info::Type_Control_Message )
	{
		if ( info.get_control_message_info() -> type
		     == Server_Raw_Control_Message_Info::Compression_OK )
		{
			this -> set_compression_level
				( info.get_control_message_info()
				  -> compression_level );

			this -> waiting_for_compression_ok = false;
		}
		else if ( info.get_control_message_info() -> type
			  == Server_Raw_Control_Message_Info
					  ::Compression_Unsupported_Warning
		       || (this -> waiting_for_compression_ok == true
			   && info.get_control_message_info() -> type
				== Server_Raw_Control_Message_Info
					   ::Illegal_Command_Form_Error) )
		{
			this -> set_compression_level( 0 );

			this -> waiting_for_compression_ok = false;
		}
	}
}
