#ifndef	   SSERVER_PLAYER_CONNECTION_H_INCLUDED
#define	   SSERVER_PLAYER_CONNECTION_H_INCLUDED

// Author:		H. Shimora
// Created:		Aug 11 1998
// Last-Modified:	Feb  5 2001
// Version:		0.20
//

//------------------------------------------------
// Change Log:
//------------------------------------------------
// version 0.10  Aug 11 1998	base version.
//
// version 0.20  Feb  5 2001	changed to Communication_Cannel base.
//
//

#include  "communication_cannel.h"
#include  "udp_connection.h"
#include  "ip_address.h"
#include  "s_basic.h"
#include  "server_raw_info.h"
#include  "sserver_param.h"
#include  "time_stamp.h"
#include  "soccer_command.h"
#include  <string>

class  SServer_Player_Connection : public Communication_Cannel<UDP_Connection>
{
public:
	enum  Parse_Policy { Strict_Parse , Fast_Parse };

public:
	static	const	int	SSERVER_DEFAULT_MAJOR_VERSION = 8;
	static	const	int	SSERVER_DEFAULT_MINOR_VERSION = 5;

	static	const	double	DEFAULT_WAIT_TIME_STEP;

protected:
	const SServer_Param &	param;

	std::string		our_team_name;
	std::string		opponent_team_name;

	S_Side_LR		our_side;
	int			self_player_number;

	long			default_wait_time;

	Parse_Policy		parse_policy;

	int			compression_level;
	bool			waiting_for_compression_ok;

	Time_Stamp		send_time;

private:
	long			wait_time() const;

	void			send_compression();

private:
	void	process_info( const Server_Raw_Info &  info );

public:
		 SServer_Player_Connection
		  ( const SServer_Param &  param ,
		    const std::string &  host = "localhost" ,
		    port_number_t  port = SServer_Param::DEFAULT_PORT ,
		    int  compression_level = 0 ,
		    long  max_response
			   = Communication_Cannel<UDP_Connection>::
				DEFAULT_MAX_RESPONCE_TIME ,
		    long  default_wait_time = -1 ,
		    Parse_Policy  p_policy = Fast_Parse );

	virtual	~SServer_Player_Connection();

	virtual	int	fd() const;

	virtual	const std::string &	get_opponent_team_name() const;

	virtual	long	default_wait() const;

	virtual	bool	send_ok() const;

	virtual	int	send_with_interval( const std::string &  str ,
					    long  interval_usec );

	virtual	int	send_init
			( const std::string &  team_name ,
			  bool  goalie ,
			  S_Side_LR *  our_side ,
			  int *  self_player_number ,
			  Play_Mode *  play_mode ,
			  int  major_version = SSERVER_DEFAULT_MAJOR_VERSION ,
			  int  minor_version = SSERVER_DEFAULT_MINOR_VERSION );


	virtual	int	send_reconnect( const std::string &  team_name ,
					int  player_number ,
					S_Side_LR *  our_side ,
					Play_Mode *  play_mode ,
					long  interval_usec = 0 );

	virtual	int	send_bye( long  interval_usec = 0 );

	virtual	int	send_command( const Soccer_Command &  command ,
				      long  interval_usec = 0 );

	virtual	int	send_reconnect( const std::string &  team_name ,
					int  player_number ,
					S_Side_LR *  our_side ,
					Play_Mode *  play_mode ,
					bool  default_wait );

	virtual	int	send_bye( bool  default_wait );

	virtual	int	send_command( const Soccer_Command &  command ,
				      bool  default_wait );

	virtual	bool	recv_info( Server_Raw_Info *  info ,
				   bool  block = false );

	virtual	bool	recv_info( Server_Raw_Info *  info ,
				   long  usec );
};


#endif	/* SSERVER_PLAYER_CONNECTION_H_INCLUDED */
