#include  "soccer_viewer_option_analyzer.h"
#include  "debug_server_connection.h"
#include  "viewer_option_descriptor.h"
#include  <iostream>
#include  <utility>

void   Soccer_Viewer_Option_Analyzer::usage( std::ostream *  ostr ) const
{
	(*ostr) << "Usage: "
		<< this -> program_name() << " [option ...]" << std::endl
		<< "Possible options are:" << std::endl
		<< "      --help" << std::endl
		<< "  -h, --host HOST" << std::endl
		<< "  -p, --port, --server-port PORT" << std::endl
		<< "      --debug-port PORT" << std::endl
		<< "      --server-protocol {1,2}" << std::endl
		<< "  -l, --log-file {FILENAME,FILENAME.{gz,bz2,Z},-}"
		<<                                                 std::endl
		<< "      --ignore-debug" << std::endl
		<< "      --interpret-client-log" << std::endl
		<< "      --left-team-config {FILENAME,-}" << std::endl
		<< "      --right-team-config {FILENAME,-}" << std::endl
		<< "      --client-log-directory PATH" << std::endl
		<< "      --client-log-viewer-command PATH" << std::endl
		<< "      --show-view-cone" << std::endl
		<< "      --hide-view-cone" << std::endl
		<< "      --view-cone-radius RADIUS" << std::endl
		<< "      --show-offside-line" << std::endl
		<< "      --hide-offside-line" << std::endl
		<< "      --show-stamina" << std::endl
		<< "      --hide-stamina" << std::endl
		<< "      --show-grid" << std::endl
		<< "      --hide-grid" << std::endl
		<< "      --auto-drop" << std::endl
		<< "      --auto-drop-on" << std::endl
		<< "      --auto-drop-off" << std::endl
		<< "      --auto-kick-off" << std::endl
		<< "      --auto-kick-off-on" << std::endl
		<< "      --auto-kick-off-off" << std::endl
		<< "      --auto-quit" << std::endl
		<< "      --auto-quit-on" << std::endl
		<< "      --auto-quit-off" << std::endl
		<< "      --auto-kill-server" << std::endl
		<< "      --auto-kill-server-on" << std::endl
		<< "      --auto-kill-server-off" << std::endl
		<< "      --time-limit LIMIT" << std::endl
		<< "      --server-process-id PID" << std::endl
		<< "      --reverse-x" << std::endl
		<< "      --reverse-y" << std::endl
		<< "      --field-grass-type {mono,lines,checker}" << std::endl
		<< "      --no-windows" << std::endl
		<< "      --default-window-size WIDTHxHEIGHT" << std::endl
		<< " and gtk options (--display etc.)" << std::endl;
}


Soccer_Viewer_Option_Analyzer::Soccer_Viewer_Option_Analyzer
			( const std::string &  default_server_host ,
			  port_number_t  default_server_port ,
			  port_number_t  default_debug_port )
	: help_( false ) ,
	  server_host_( default_server_host ) ,
	  server_port_( default_server_port ) ,
	  debug_port_( default_debug_port ) ,
	  server_protocol_version_( 2 ) ,
	  log_file_source_( false ) ,
	  log_file_name_( "" ) ,
	  receive_debug_( true ) ,
	  interpret_client_log_( false ) ,
	  have_left_team_config_( false ) ,
	  left_team_config_file_name_( "" ) ,
	  have_right_team_config_( false ) ,
	  right_team_config_file_name_( "" ) ,
	  client_log_directory_( "" ) ,
	  client_log_viewer_command_( "" ) ,
	  draw_player_view_cone_( true ) ,
	  player_view_cone_radius_( Field_Canvas::DEFAULT_VIEW_CONE_RADIUS ) ,
	  draw_offside_line_( false ) ,
	  draw_stamina_( false ) ,
	  draw_grid_( false ) ,
	  grid_width_( 5.0 ) ,
	  auto_drop_( true ) ,
	  auto_kick_off_( false ) ,
	  auto_quit_( false ) ,
	  auto_kill_server_( false ) ,
	  time_limit_( -1 ) ,
	  server_process_id_( -1 ) ,
	  reverse_x_( false ) ,
	  reverse_y_( false ) ,
	  field_grass_type_( Field_Canvas::DEFAULT_GRASS_TYPE ) ,
	  no_windows_( false ) ,
	  default_window_size_( std::pair<int, int>( 0 , 0 ) )
{
	//
	// Add Options
	//
	this -> add_long_option_flag_on(      "--help"       , &help_ );
	this -> add_option_string      ( 'h', "--host"       , &server_host_ );
	this -> add_long_option_string (      "--server-host", &server_host_ );
	this -> add_option_port_number ( 'p', "--port"       , &server_port_ );
	this -> add_long_option_port_number(  "--server-port", &server_port_ );
	this -> add_long_option_port_number(  "--debug-port" , &debug_port_ );

	this -> add_long_option_integer(      "--server-protocol",
					      &server_protocol_version_ );
	this -> add_option             ( 'l', "--log-file",
					 new File_Name_Option_Descriptor
					     ( &log_file_name_ ,
					       &log_file_source_ ) );

	this -> add_long_option_flag_off(     "--ignore-debug" ,
					      &receive_debug_ );
	this -> add_long_option_flag_on(      "--interpret-client-log" ,
					      &interpret_client_log_ );

	this -> add_long_option        ( "--left-team-config",
					 new File_Name_Option_Descriptor
					     ( &left_team_config_file_name_ ,
					       &have_left_team_config_ ) );
	this -> add_long_option        ( "--right-team-config" ,
					 new File_Name_Option_Descriptor
					     ( &right_team_config_file_name_ ,
					       &have_right_team_config_ ) );

	this -> add_long_option        ( "--client-log-viewer-command",
					 new File_Name_Option_Descriptor
					     ( &client_log_viewer_command_ ) );
	this -> add_long_option        ( "--client-log-directory",
					 new File_Name_Option_Descriptor
					     ( &client_log_directory_ ) );

	this -> add_long_option_flag_on(      "--show-view-cone",
					      &draw_player_view_cone_ );
	this -> add_long_option_flag_off(     "--hide-view-cone",
					      &draw_player_view_cone_ );
	this -> add_long_option_float(        "--view-cone-radius",
					      &player_view_cone_radius_ );
	this -> add_long_option_flag_on(      "--show-offside-line",
					      &draw_offside_line_ );
	this -> add_long_option_flag_off(     "--hide-offside-line",
					      &draw_offside_line_ );
	this -> add_long_option_flag_on(      "--show-stamina",
					      &draw_stamina_ );
	this -> add_long_option_flag_off(     "--hide-stamina",
					      &draw_stamina_ );
	this -> add_long_option_flag_on(      "--show-grid",
					      &draw_grid_ );
	this -> add_long_option_flag_off(     "--hide-grid",
					      &draw_grid_ );
	this -> add_long_option_float   (     "--grid-width",
					      &grid_width_ );

	this -> add_long_option_flag_on (     "--auto-drop" ,
					      &auto_drop_ );
	this -> add_long_option_flag_on (     "--auto-drop-on" ,
					      &auto_drop_ );
	this -> add_long_option_flag_off(     "--auto-drop-off" ,
					      &auto_drop_ );

	this -> add_long_option_flag_on (     "--auto-kick-off" ,
					      &auto_kick_off_ );
	this -> add_long_option_flag_on (     "--auto-kick-off-on" ,
					      &auto_kick_off_ );
	this -> add_long_option_flag_off(     "--auto-kick-off-off" ,
					      &auto_kick_off_ );

	this -> add_long_option_flag_on (     "--auto-quit" ,
					      &auto_quit_ );
	this -> add_long_option_flag_on (     "--auto-quit-on" ,
					      &auto_quit_ );
	this -> add_long_option_flag_off(     "--auto-quit-off" ,
					      &auto_quit_ );

	this -> add_long_option_flag_on (     "--auto-kill-server" ,
					      &auto_kill_server_ );
	this -> add_long_option_flag_on (     "--auto-kill-server-on" ,
					      &auto_kill_server_ );
	this -> add_long_option_flag_off(     "--auto-kill-server-off" ,
					      &auto_kill_server_ );

	this -> add_long_option_long_integer( "--time-limit",
					      &time_limit_ );

	this -> add_long_option_flag_on(      "--reverse-x" ,
					      &reverse_x_ );
	this -> add_long_option_flag_on(      "--reverse-y" ,
					      &reverse_y_ );
	this -> add_long_option
		( "--field-grass-type" ,
		  new Field_Grass_Type_Option_Descriptor
					      ( &field_grass_type_ ) );

	this -> add_long_option_integer(      "--server-process-id",
					      &server_process_id_ );

	this -> add_long_option_flag_on(      "--no-windows" ,
					      &no_windows_ );

	this -> add_long_option
		( "--default-window-size" ,
		  new Window_Size_Option_Descriptor( &default_window_size_ ) );
}

Soccer_Viewer_Option_Analyzer::~Soccer_Viewer_Option_Analyzer()
{
}

void   Soccer_Viewer_Option_Analyzer::after_analyze_hook()
{
	if ( this -> rest().size() != 0 )
	{
		this -> log_file_source_ = true;
		this -> log_file_name_ = *(this -> rest().rbegin());

		this -> rest().clear();
	}
}

bool   Soccer_Viewer_Option_Analyzer::help() const
{
	return( this -> help_ );
}

std::string  Soccer_Viewer_Option_Analyzer::server_host() const
{
	return( this -> server_host_ );
}

port_number_t  Soccer_Viewer_Option_Analyzer::server_port() const
{
	return( this -> server_port_ );
}

port_number_t  Soccer_Viewer_Option_Analyzer::debug_port() const
{
	if ( this -> debug_port_ == 0 )
	{
		return( this -> server_port()
			  + Debug_Server_Connection::DEFAULT_PORT_OFFSET );
	}
	else
	{
		return( this -> debug_port_ );
	}
}

int    Soccer_Viewer_Option_Analyzer::server_protocol_version() const
{
	return( this -> server_protocol_version_ );
}

bool   Soccer_Viewer_Option_Analyzer::log_file_source() const
{
	return( this -> log_file_source_ );
}

std::string  Soccer_Viewer_Option_Analyzer::log_file_name() const
{
	return( this -> log_file_name_ );
}

bool   Soccer_Viewer_Option_Analyzer::receive_debug() const
{
	return( this -> receive_debug_ );
}

bool   Soccer_Viewer_Option_Analyzer::interpret_client_log() const
{
	return( this -> interpret_client_log_ );
}

bool   Soccer_Viewer_Option_Analyzer::have_left_team_config() const
{
	return( this -> have_left_team_config_ );
}

std::string  Soccer_Viewer_Option_Analyzer::left_team_config_file_name() const
{
	return( this -> left_team_config_file_name_ );
}

bool   Soccer_Viewer_Option_Analyzer::have_right_team_config() const
{
	return( this -> have_right_team_config_ );
}

std::string  Soccer_Viewer_Option_Analyzer::right_team_config_file_name() const
{
	return( this -> right_team_config_file_name_ );
}

std::string  Soccer_Viewer_Option_Analyzer::client_log_directory() const
{
	return( this -> client_log_directory_ );
}

std::string  Soccer_Viewer_Option_Analyzer::client_log_viewer_command() const
{
	return( this -> client_log_viewer_command_ );
}

bool   Soccer_Viewer_Option_Analyzer::auto_drop() const
{
	return( this -> auto_drop_ );
}

bool   Soccer_Viewer_Option_Analyzer::draw_player_view_cone() const
{
	return( this -> draw_player_view_cone_ );
}

double  Soccer_Viewer_Option_Analyzer::player_view_cone_radius() const
{
	return( this -> player_view_cone_radius_ );
}

bool   Soccer_Viewer_Option_Analyzer::draw_offside_line() const
{
	return( this -> draw_offside_line_ );
}

bool   Soccer_Viewer_Option_Analyzer::draw_stamina() const
{
	return( this -> draw_stamina_ );
}

bool   Soccer_Viewer_Option_Analyzer::draw_grid() const
{
	return( this -> draw_grid_ );
}

double  Soccer_Viewer_Option_Analyzer::grid_width() const
{
	return( this -> grid_width_ );
}

bool   Soccer_Viewer_Option_Analyzer::auto_kick_off() const
{
	return( this -> auto_kick_off_ );
}

bool   Soccer_Viewer_Option_Analyzer::auto_quit() const
{
	return( this -> auto_quit_ );
}

bool   Soccer_Viewer_Option_Analyzer::auto_kill_server() const
{
	return( this -> auto_kill_server_ );
}

long   Soccer_Viewer_Option_Analyzer::time_limit() const
{
	return( this -> time_limit_ );
}

pid_t  Soccer_Viewer_Option_Analyzer::server_process_id() const
{
	return( static_cast<pid_t>( this -> server_process_id_ ) );
}

bool   Soccer_Viewer_Option_Analyzer::reverse_x() const
{
	return( this -> reverse_x_ );
}

bool   Soccer_Viewer_Option_Analyzer::reverse_y() const
{
	return( this -> reverse_y_ );
}

Field_Canvas::Field_Grass_Type  Soccer_Viewer_Option_Analyzer
						::field_grass_type() const
{
	return( this -> field_grass_type_ );
}

bool  Soccer_Viewer_Option_Analyzer::no_windows() const
{
	return( this -> no_windows_ );
}

std::pair<int, int>  Soccer_Viewer_Option_Analyzer::default_window_size()
	const
{
	return( this -> default_window_size_ );
}
