#include "view_config.h"
#include  <cassert>

using namespace std;


const double	View_Config::DEFAULT_PLAYER_SIZE      = 1.3;
const double	View_Config::DEFAULT_BALL_SIZE        = 0.8;
const double	View_Config::DEFAULT_MAGNIFY          = 8.0;

View_Config::View_Point::View_Point()
	: type( View_Point::Point ) ,
	  coordinate( 0.0 , 0.0 )
{
}

View_Config::Debug_View_Config::Debug_View_Config()
	: debug_ball( false ) ,
	  debug_self( false ) ,
	  debug_players( false ) ,
	  debug_comment( false )
{
}

void   View_Config::Debug_View_Config::set_all()
{
	debug_ball = true;
	debug_self = true;
	debug_players = true;
	debug_comment = true;
}

void   View_Config::Debug_View_Config::reset_all()
{
	debug_ball = false;
	debug_self = false;
	debug_players = false;
	debug_comment = false;
}


View_Config::View_Config( bool  auto_drop ,
			  bool  auto_kick_off ,
			  bool  auto_quit ,
			  bool  auto_kill_server ,
			  bool  player_view_cone ,
			  double  player_view_cone_radius ,
			  bool  offside_line ,
			  bool  stamina ,
			  bool  grid ,
			  double  grid_width ,
			  bool  reverse_x ,
			  bool  reverse_y ,
			  const Field_Canvas::Field_Grass_Type &  grass_type )
	: monitor_view( true ) , player_view_cone( player_view_cone ) ,
	  player_view_cone_radius( player_view_cone_radius ) ,
	  offside_line( offside_line ) , stamina( stamina ) ,
	  view_point() ,
	  auto_adjust_magnify( true ) , field_magnify( DEFAULT_MAGNIFY ) ,
	  reverse_x( reverse_x ) , reverse_y( reverse_y ) ,
	  field_grass_type( grass_type ) ,
	  player_size( DEFAULT_PLAYER_SIZE ) , ball_size( DEFAULT_BALL_SIZE ) ,
	  draw_grid( grid ) , grid_width( grid_width ) ,
	  draw_circle( false ) , circle_radius( 3.0 ) ,
	  use_team_name( true ) ,
	  auto_drop( auto_drop ) ,
	  auto_kick_off( auto_kick_off ) ,
	  auto_quit( auto_quit ) ,
	  auto_kill_server( auto_kill_server )
{
}

View_Config::~View_Config()
{
}

View_Config::Debug_View_Config &  View_Config::debug_config
						( const S_Side_LR &  side ,
						  int  player_number )
{
	assert( side != S_Side_LR::Unknown
		&& (1 <= player_number && player_number <= MAX_PLAYER) );

	if ( side == S_Side_LR::Right_Side )
	{
		return( right_player[ player_number - 1 ] );
	}
	else
	{
		return( left_player[ player_number - 1 ] );
	}
}

const View_Config::Debug_View_Config &  View_Config::debug_config
						    ( const S_Side_LR &  side ,
						      int  player_number )
	const
{
	return( const_cast<View_Config *>(this)
		  -> debug_config( side , player_number ) );
}
