#include  "team_config.h"
#include  "s_basic.h"
#include  "compat_stringstream.h"
#include  <string>
#include  <cstdio>
#include  <cstdlib>
#include  <ctype.h>
#include  <iostream>

Team_Config::Team_Config()
	: player( MAX_PLAYER ) , have_player_config( false )
{
	this -> clear();
}

Team_Config::~Team_Config()
{
}

void   Team_Config::clear()
{
	have_player_config = false;

	for ( size_t  i = 0  ;  i < player.size()  ;  i ++ )
	{
		player[i].have_name  = false;
		player[i].have_color = false;
	}
}

void   Team_Config::read_config( std::istream &  input )
{
	for(;;)
	{
		std::string	line;
		std::getline( input , line );

		if ( ! input || input.eof() )
		{
			break;
		}

		if ( line.find( '#' ) != std::string::npos )
		{
			line.resize( line.find( '#' ) );
		}


		if ( line.find_first_not_of( " \t" ) == std::string::npos )
		{
			continue;
		}


		compat_istringstream	config_line( line );

		int	number = 0;
		config_line >> number;

		if ( ! config_line
		  || (number < 1 || MAX_PLAYER < number) )
		{
			std::cerr << "team config: Bad player number."
				  << std::endl;
			this -> clear();
			return;
		}

		config_line >> std::ws;

		if ( ! config_line || config_line.eof() )
		{
			std::cerr
				<< "team config: No configuration for player "
				<< number << "." << std::endl;
			this -> clear();
			return;
		}


		for(;;)
		{
			std::string	config_name;
			config_line >> std::ws >> config_name >> std::ws;


			bool		have_value = false;
			std::string	config_value;
			bool		in_quote = false;
			for(;;)
			{
				char	ch;
				config_line.get( ch );

				if ( config_line.eof() )
				{
					break;
					return;
				}

				if ( in_quote )
				{
					if ( ch == '"' )
					{
						in_quote = false;
					}
					else
					{
						have_value = true;
						config_value += ch;
					}
				}
				else
				{
					if ( ch == '"' )
					{
						have_value = true;
						in_quote = true;
					}
					else if ( isspace( ch ) )
					{
						break;
					}
					else
					{
						have_value = true;
						config_value += ch;
					}
				}
			}


			if ( ! have_value )
			{
				std::cerr << "team config: No value for \""
					  << config_name << "\"." << std::endl;
				this -> clear();
				return;
			}

			if ( config_name == "name" )
			{
				player[number - 1].have_name = true;
				player[number - 1].name = config_value;
				have_player_config = true;
			}
			else if ( config_name == "color" )
			{
				player[number - 1].have_color = true;
				player[number - 1].color = config_value;
				have_player_config = true;
			}
			else
			{
				std::cerr
				     << "team config: No such configration \""
				     << config_name << "\"." << std::endl;
				this -> clear();
				return;
			}

			config_line >> std::ws;

			if ( config_line.eof() )
			{
				break;
			}
		}
	}
}
