#ifndef	   VIEWER_OPTION_DESCRIPTOR_H_INCLUDED
#define	   VIEWER_OPTION_DESCRIPTOR_H_INCLUDED

#include  "option_analyzer.h"
#include  "field_canvas.h"
#include  <iostream>
#include  <utility>
#include  <cstdio>

//
// local class for --default-window-size option
//
class  Window_Size_Option_Descriptor
	: public Option_Analyzer::Option_Descriptor
{
protected:
	mutable	std::pair<int, int> *	size;

public:
	Window_Size_Option_Descriptor( std::pair<int, int> *  s )
		: size( s ) {}

	virtual	~Window_Size_Option_Descriptor() {}

	virtual	int	execute( const std::vector<std::string> &  arg ) const
	{
		if ( arg.size() < 1 )
		{
			return( -1 );
		}

		int	window_width  = 0;
		int	window_height = 0;
		int	n = 0;

		if ( std::sscanf( arg[0].c_str() ,
				  "%dx%d%n" ,
				  &window_width , &window_height , &n ) == 2
		     && arg[0][n] == '\0' )
		{
			*(this -> size) = std::pair<int, int>( window_width ,
							       window_height );

			return( 1 );
		}
		else
		{
			std::cerr << "Bad window size \"" << arg[0] << "\"."
				  << std::endl;

			std::cerr << std::endl;

			return( -1 );
		}
	}
};


//
// local class for --log-file option
//
class  File_Name_Option_Descriptor
	: public Option_Analyzer::String_Option_Descriptor
{
protected:
	bool *	file_specified;

public:
	File_Name_Option_Descriptor( std::string *  file_name ,
				     bool *  file_specified
						     = static_cast<bool *>(0) )
		: Option_Analyzer::String_Option_Descriptor( file_name ) ,
		  file_specified( file_specified ) {}

	virtual	~File_Name_Option_Descriptor() {}

	virtual	int	execute( const std::vector<std::string> &  arg ) const
	{
		if ( this -> file_specified )
		{
			*(this -> file_specified) = true;
		}

		return( Option_Analyzer::String_Option_Descriptor
			::execute( arg ) );
	}
};


//
// local class for --field-grass-type
//
class  Field_Grass_Type_Option_Descriptor
	: public Option_Analyzer::Option_Descriptor
{
protected:
	Field_Canvas::Field_Grass_Type	*	type;

public:
	Field_Grass_Type_Option_Descriptor
			( Field_Canvas::Field_Grass_Type *  type )
		: type( type ) {}

	virtual	~Field_Grass_Type_Option_Descriptor() {}

	virtual	int	execute( const std::vector<std::string> &  arg ) const
	{
		if ( arg.size() < 1 )
		{
			return( -1 );
		}

		if ( arg[0] == "mono" )
		{
			*(this -> type) = Field_Canvas::Field_Grass_Type_Mono;
		}
		else if ( arg[0] == "lines" )
		{
			*(this -> type) = Field_Canvas::Field_Grass_Type_Lines;
		}
		else if ( arg[0] == "checker" )
		{
			*(this -> type)
				= Field_Canvas::Field_Grass_Type_Chacker;
		}
		else
		{
			std::cerr << "bad field grass type \""
				  << arg[0] << "\"." << std::endl;

			return( -1 );
		}

		return( 1 );
	}
};


#endif	/* VIEWER_OPTION_DESCRIPTOR_H_INCLUDED */
